@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifndef APP_APP_OPTIONS_H_INCLUDED
 #define APP_APP_OPTIONS_H_INCLUDED
@@ -30,6 +19,12 @@ namespace app {
 
 class AppOptions {
 public:
+  enum VerboseLevel {
+    kNoVerbose,
+    kVerbose,
+    kHighlyVerbose,
+  };
+
   typedef base::ProgramOptions PO;
   typedef PO::Option Option;
   typedef PO::ValueList ValueList;
@@ -38,7 +33,7 @@ public:
 
   bool startUI() const { return m_startUI; }
   bool startShell() const { return m_startShell; }
-  bool verbose() const { return m_verboseEnabled; }
+  VerboseLevel verboseLevel() const { return m_verboseLevel; }
 
   const std::string& paletteFileName() const { return m_paletteFileName; }
 
@@ -49,16 +44,29 @@ public:
   // Export options
   const Option& saveAs() const { return m_saveAs; }
   const Option& scale() const { return m_scale; }
+  const Option& shrinkTo() const { return m_shrinkTo; }
   const Option& data() const { return m_data; }
+  const Option& format() const { return m_format; }
   const Option& sheet() const { return m_sheet; }
   const Option& sheetWidth() const { return m_sheetWidth; }
   const Option& sheetHeight() const { return m_sheetHeight; }
+  const Option& sheetType() const { return m_sheetType; }
   const Option& sheetPack() const { return m_sheetPack; }
   const Option& splitLayers() const { return m_splitLayers; }
-  const Option& importLayer() const { return m_importLayer; }
+  const Option& layer() const { return m_layer; }
+  const Option& allLayers() const { return m_allLayers; }
+  const Option& frameTag() const { return m_frameTag; }
+  const Option& frameRange() const { return m_frameRange; }
   const Option& ignoreEmpty() const { return m_ignoreEmpty; }
+  const Option& borderPadding() const { return m_borderPadding; }
+  const Option& shapePadding() const { return m_shapePadding; }
+  const Option& innerPadding() const { return m_innerPadding; }
   const Option& trim() const { return m_trim; }
+  const Option& crop() const { return m_crop; }
   const Option& filenameFormat() const { return m_filenameFormat; }
+  const Option& script() const { return m_script; }
+  const Option& listLayers() const { return m_listLayers; }
+  const Option& listTags() const { return m_listTags; }
 
   bool hasExporterParams() const;
 
@@ -70,7 +78,7 @@ private:
   base::ProgramOptions m_po;
   bool m_startUI;
   bool m_startShell;
-  bool m_verboseEnabled;
+  VerboseLevel m_verboseLevel;
   std::string m_paletteFileName;
 
   Option& m_palette;
@@ -78,18 +86,32 @@ private:
   Option& m_batch;
   Option& m_saveAs;
   Option& m_scale;
+  Option& m_shrinkTo;
   Option& m_data;
+  Option& m_format;
   Option& m_sheet;
   Option& m_sheetWidth;
   Option& m_sheetHeight;
+  Option& m_sheetType;
   Option& m_sheetPack;
   Option& m_splitLayers;
-  Option& m_importLayer;
+  Option& m_layer;
+  Option& m_allLayers;
+  Option& m_frameTag;
+  Option& m_frameRange;
   Option& m_ignoreEmpty;
+  Option& m_borderPadding;
+  Option& m_shapePadding;
+  Option& m_innerPadding;
   Option& m_trim;
+  Option& m_crop;
   Option& m_filenameFormat;
+  Option& m_script;
+  Option& m_listLayers;
+  Option& m_listTags;
 
   Option& m_verbose;
+  Option& m_debug;
   Option& m_help;
   Option& m_version;
 
