@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -25,10 +14,12 @@
 #include "app/app.h"
 #include "app/commands/command.h"
 #include "app/commands/params.h"
+#include "app/modules/palettes.h"
 #include "app/ui/color_bar.h"
+#include "doc/palette.h"
 
 namespace app {
-  
+
 class ChangeColorCommand : public Command
 {
   enum Change {
@@ -48,9 +39,9 @@ public:
   ChangeColorCommand();
 
 protected:
-  void onLoadParams(Params* params);
-  void onExecute(Context* context);
-  std::string onGetFriendlyName() const;
+  void onLoadParams(const Params& params) override;
+  void onExecute(Context* context) override;
+  std::string onGetFriendlyName() const override;
 };
 
 ChangeColorCommand::ChangeColorCommand()
@@ -62,13 +53,13 @@ ChangeColorCommand::ChangeColorCommand()
   m_change = None;
 }
 
-void ChangeColorCommand::onLoadParams(Params* params)
+void ChangeColorCommand::onLoadParams(const Params& params)
 {
-  std::string target = params->get("target");
+  std::string target = params.get("target");
   if (target == "foreground") m_background = false;
   else if (target == "background") m_background = true;
 
-  std::string change = params->get("change");
+  std::string change = params.get("change");
   if (change == "increment-index") m_change = IncrementIndex;
   else if (change == "decrement-index") m_change = DecrementIndex;
 }
@@ -83,24 +74,18 @@ void ChangeColorCommand::onExecute(Context* context)
     case None:
       // do nothing
       break;
-    case IncrementIndex:
-      if (color.getType() == app::Color::IndexType) {
-        int index = color.getIndex();
-        if (index < 255)        // TODO use sprite palette limit
-          color = app::Color::fromIndex(index+1);
-      }
-      else
-        color = app::Color::fromIndex(0);
+    case IncrementIndex: {
+      int index = color.getIndex();
+      if (index < get_current_palette()->size()-1)
+        color = app::Color::fromIndex(index+1);
       break;
-    case DecrementIndex:
-      if (color.getType() == app::Color::IndexType) {
-        int index = color.getIndex();
-        if (index > 0)
-          color = app::Color::fromIndex(index-1);
-      }
-      else
-        color = app::Color::fromIndex(0);
+    }
+    case DecrementIndex: {
+      int index = color.getIndex();
+      if (index > 0)
+        color = app::Color::fromIndex(index-1);
       break;
+    }
   }
 
   if (m_background)
