@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2015  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -22,19 +11,20 @@
 
 #include "app/commands/command.h"
 #include "app/commands/params.h"
-#include "app/find_widget.h"
-#include "app/load_widget.h"
+#include "app/loop_tag.h"
 #include "app/modules/editors.h"
 #include "app/modules/gui.h"
 #include "app/ui/editor/editor.h"
-#include "raster/sprite.h"
+#include "doc/frame_tag.h"
+#include "doc/sprite.h"
 #include "ui/window.h"
 
-#include <allegro/unicode.h>
+#include "goto_frame.xml.h"
 
 namespace app {
 
 using namespace ui;
+using namespace doc;
 
 class GotoCommand : public Command {
 protected:
@@ -51,7 +41,7 @@ protected:
     current_editor->setFrame(onGetFrame(current_editor));
   }
 
-  virtual FrameNumber onGetFrame(Editor* editor) = 0;
+  virtual frame_t onGetFrame(Editor* editor) = 0;
 };
 
 class GotoFirstFrameCommand : public GotoCommand {
@@ -62,8 +52,8 @@ public:
   Command* clone() const override { return new GotoFirstFrameCommand(*this); }
 
 protected:
-  FrameNumber onGetFrame(Editor* editor) override {
-    return FrameNumber(0);
+  frame_t onGetFrame(Editor* editor) override {
+    return 0;
   }
 };
 
@@ -75,13 +65,11 @@ public:
   Command* clone() const override { return new GotoPreviousFrameCommand(*this); }
 
 protected:
-  FrameNumber onGetFrame(Editor* editor) override {
-    FrameNumber frame = editor->frame();
+  frame_t onGetFrame(Editor* editor) override {
+    frame_t frame = editor->frame();
+    frame_t last = editor->sprite()->lastFrame();
 
-    if (frame > FrameNumber(0))
-      return frame.previous();
-    else
-      return editor->sprite()->lastFrame();
+    return (frame > 0 ? frame-1: last);
   }
 };
 
@@ -92,12 +80,45 @@ public:
   Command* clone() const override { return new GotoNextFrameCommand(*this); }
 
 protected:
-  FrameNumber onGetFrame(Editor* editor) override {
-    FrameNumber frame = editor->frame();
-    if (frame < editor->sprite()->lastFrame())
-      return frame.next();
-    else
-      return FrameNumber(0);
+  frame_t onGetFrame(Editor* editor) override {
+    frame_t frame = editor->frame();
+    frame_t last = editor->sprite()->lastFrame();
+
+    return (frame < last ? frame+1: 0);
+  }
+};
+
+class GotoNextFrameWithSameTagCommand : public GotoCommand {
+public:
+  GotoNextFrameWithSameTagCommand() : GotoCommand("GotoNextFrameWithSameTag",
+                                                  "Go to Next Frame with same tag") { }
+  Command* clone() const override { return new GotoNextFrameWithSameTagCommand(*this); }
+
+protected:
+  frame_t onGetFrame(Editor* editor) override {
+    frame_t frame = editor->frame();
+    FrameTag* tag = get_animation_tag(editor->sprite(), frame);
+    frame_t first = (tag ? tag->fromFrame(): 0);
+    frame_t last = (tag ? tag->toFrame(): editor->sprite()->lastFrame());
+
+    return (frame < last ? frame+1: first);
+  }
+};
+
+class GotoPreviousFrameWithSameTagCommand : public GotoCommand {
+public:
+  GotoPreviousFrameWithSameTagCommand() : GotoCommand("GotoPreviousFrameWithSameTag",
+                                                      "Go to Previous Frame with same tag") { }
+  Command* clone() const override { return new GotoPreviousFrameWithSameTagCommand(*this); }
+
+protected:
+  frame_t onGetFrame(Editor* editor) override {
+    frame_t frame = editor->frame();
+    FrameTag* tag = get_animation_tag(editor->sprite(), frame);
+    frame_t first = (tag ? tag->fromFrame(): 0);
+    frame_t last = (tag ? tag->toFrame(): editor->sprite()->lastFrame());
+
+    return (frame > first ? frame-1: last);
   }
 };
 
@@ -108,7 +129,7 @@ public:
   Command* clone() const override { return new GotoLastFrameCommand(*this); }
 
 protected:
-  FrameNumber onGetFrame(Editor* editor) override {
+  frame_t onGetFrame(Editor* editor) override {
     return editor->sprite()->lastFrame();
   }
 };
@@ -121,29 +142,26 @@ public:
   Command* clone() const override { return new GotoFrameCommand(*this); }
 
 protected:
-  void onLoadParams(Params* params) override
-  {
-    std::string frame = params->get("frame");
-    if (!frame.empty()) m_frame = ustrtol(frame.c_str(), NULL, 10);
+  void onLoadParams(const Params& params) override {
+    std::string frame = params.get("frame");
+    if (!frame.empty()) m_frame = strtol(frame.c_str(), NULL, 10);
     else m_frame = 0;
   }
 
-  FrameNumber onGetFrame(Editor* editor) override {
+  frame_t onGetFrame(Editor* editor) override {
     if (m_frame == 0) {
-      base::UniquePtr<Window> window(app::load_widget<Window>("goto_frame.xml", "goto_frame"));
-      Widget* frame = app::find_widget<Widget>(window, "frame");
-      Widget* ok = app::find_widget<Widget>(window, "ok");
+      app::gen::GotoFrame window;
 
-      frame->setTextf("%d", editor->frame()+1);
+      window.frame()->setTextf("%d", editor->frame()+1);
 
-      window->openWindowInForeground();
-      if (window->getKiller() != ok)
+      window.openWindowInForeground();
+      if (window.closer() != window.ok())
         return editor->frame();
 
-      m_frame = frame->getTextInt();
+      m_frame = window.frame()->textInt();
     }
 
-    return MID(FrameNumber(0), FrameNumber(m_frame-1), editor->sprite()->lastFrame());
+    return MID(0, m_frame-1, editor->sprite()->lastFrame());
   }
 
 private:
@@ -172,6 +190,16 @@ Command* CommandFactory::createGotoLastFrameCommand()
   return new GotoLastFrameCommand;
 }
 
+Command* CommandFactory::createGotoNextFrameWithSameTagCommand()
+{
+  return new GotoNextFrameWithSameTagCommand;
+}
+
+Command* CommandFactory::createGotoPreviousFrameWithSameTagCommand()
+{
+  return new GotoPreviousFrameWithSameTagCommand;
+}
+
 Command* CommandFactory::createGotoFrameCommand()
 {
   return new GotoFrameCommand;
