@@ -1,31 +1,20 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2015  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include "app/app.h"
 #include "app/commands/command.h"
 #include "app/commands/params.h"
 #include "app/context_access.h"
 #include "app/modules/editors.h"
-#include "app/settings/document_settings.h"
-#include "app/settings/settings.h"
+#include "app/pref/preferences.h"
 #include "app/ui/editor/editor.h"
 #include "base/convert_to.h"
 #include "ui/view.h"
@@ -50,10 +39,10 @@ public:
   Command* clone() const override { return new ScrollCommand(*this); }
 
 protected:
-  void onLoadParams(Params* params);
-  bool onEnabled(Context* context);
-  void onExecute(Context* context);
-  std::string onGetFriendlyName() const;
+  void onLoadParams(const Params& params) override;
+  bool onEnabled(Context* context) override;
+  void onExecute(Context* context) override;
+  std::string onGetFriendlyName() const override;
 
 private:
   Direction m_direction;
@@ -68,15 +57,15 @@ ScrollCommand::ScrollCommand()
 {
 }
 
-void ScrollCommand::onLoadParams(Params* params)
+void ScrollCommand::onLoadParams(const Params& params)
 {
-  std::string direction = params->get("direction");
+  std::string direction = params.get("direction");
   if (direction == "left") m_direction = Left;
   else if (direction == "right") m_direction = Right;
   else if (direction == "up") m_direction = Up;
   else if (direction == "down") m_direction = Down;
 
-  std::string units = params->get("units");
+  std::string units = params.get("units");
   if (units == "pixel") m_units = Pixel;
   else if (units == "tile-width") m_units = TileWidth;
   else if (units == "tile-height") m_units = TileHeight;
@@ -86,7 +75,7 @@ void ScrollCommand::onLoadParams(Params* params)
   else if (units == "viewport-width") m_units = ViewportWidth;
   else if (units == "viewport-height") m_units = ViewportHeight;
 
-  int quantity = params->get_as<int>("quantity");
+  int quantity = params.get_as<int>("quantity");
   m_quantity = std::max<int>(1, quantity);
 }
 
@@ -98,13 +87,12 @@ bool ScrollCommand::onEnabled(Context* context)
 
 void ScrollCommand::onExecute(Context* context)
 {
-  IDocumentSettings* docSettings = context->settings()->getDocumentSettings(context->activeDocument());
+  DocumentPreferences& docPref = Preferences::instance().document(context->activeDocument());
   ui::View* view = ui::View::getView(current_editor);
-  gfx::Rect vp = view->getViewportBounds();
-  gfx::Point scroll = view->getViewScroll();
-  gfx::Rect gridBounds = docSettings->getGridBounds();
-  int dx = 0;
-  int dy = 0;
+  gfx::Rect vp = view->viewportBounds();
+  gfx::Point scroll = view->viewScroll();
+  gfx::Rect gridBounds = docPref.grid.bounds();
+  gfx::Point delta(0, 0);
   int pixels = 0;
 
   switch (m_units) {
@@ -118,13 +106,13 @@ void ScrollCommand::onExecute(Context* context)
       pixels = gridBounds.h;
       break;
     case ZoomedPixel:
-      pixels = 1 << current_editor->zoom();
+      pixels = current_editor->zoom().apply(1);
       break;
     case ZoomedTileWidth:
-      pixels = gridBounds.w << current_editor->zoom();
+      pixels = current_editor->zoom().apply(gridBounds.w);
       break;
     case ZoomedTileHeight:
-      pixels = gridBounds.h << current_editor->zoom();
+      pixels = current_editor->zoom().apply(gridBounds.h);
       break;
     case ViewportWidth:
       pixels = vp.h;
@@ -135,13 +123,13 @@ void ScrollCommand::onExecute(Context* context)
   }
 
   switch (m_direction) {
-    case Left:  dx = -m_quantity * pixels; break;
-    case Right: dx = +m_quantity * pixels; break;
-    case Up:    dy = -m_quantity * pixels; break;
-    case Down:  dy = +m_quantity * pixels; break;
+    case Left:  delta.x = -m_quantity * pixels; break;
+    case Right: delta.x = +m_quantity * pixels; break;
+    case Up:    delta.y = -m_quantity * pixels; break;
+    case Down:  delta.y = +m_quantity * pixels; break;
   }
 
-  current_editor->setEditorScroll(scroll.x+dx, scroll.y+dy, true);
+  current_editor->setEditorScroll(scroll+delta);
 }
 
 std::string ScrollCommand::onGetFriendlyName() const
