@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -23,11 +12,13 @@
 #include "app/commands/filters/filter_preview.h"
 
 #include "app/commands/filters/filter_manager_impl.h"
-#include "raster/sprite.h"
+#include "app/modules/editors.h"
+#include "app/ui/editor/editor.h"
+#include "doc/layer.h"
+#include "doc/sprite.h"
 #include "ui/manager.h"
 #include "ui/message.h"
 #include "ui/widget.h"
-#include "app/util/render.h"
 
 namespace app {
 
@@ -75,14 +66,16 @@ bool FilterPreview::onProcessMessage(Message* msg)
   switch (msg->type()) {
 
     case kOpenMessage:
-      RenderEngine::setPreviewImage(
+      current_editor->renderEngine().setPreviewImage(
         m_filterMgr->layer(),
         m_filterMgr->frame(),
-        m_filterMgr->destinationImage());
+        m_filterMgr->destinationImage(),
+        m_filterMgr->position(),
+        static_cast<doc::LayerImage*>(m_filterMgr->layer())->blendMode());
       break;
 
     case kCloseMessage:
-      RenderEngine::setPreviewImage(NULL, FrameNumber(0), NULL);
+      current_editor->renderEngine().removePreviewImage();
 
       // Stop the preview timer.
       m_timer.stop();
