@@ -1,25 +1,20 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifndef APP_CONTEXT_FLAGS_H_INCLUDED
 #define APP_CONTEXT_FLAGS_H_INCLUDED
 #pragma once
 
+#include "base/ints.h"
+
+namespace doc {
+  class Site;
+}
+
 namespace app {
 
   class Context;
@@ -38,8 +33,8 @@ namespace app {
       ActiveDocumentIsWritable    = 1 << 8,
       ActiveLayerIsImage          = 1 << 9,
       ActiveLayerIsBackground     = 1 << 10,
-      ActiveLayerIsReadable       = 1 << 11,
-      ActiveLayerIsWritable       = 1 << 12,
+      ActiveLayerIsVisible        = 1 << 11,
+      ActiveLayerIsEditable       = 1 << 12,
     };
 
     ContextFlags();
@@ -48,6 +43,8 @@ namespace app {
     void update(Context* context);
 
   private:
+    void updateFlagsFromSite(const doc::Site& site);
+
     uint32_t m_flags;
   };
 
