@@ -1,46 +1,36 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2015  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifndef APP_DOCUMENT_H_INCLUDED
 #define APP_DOCUMENT_H_INCLUDED
 #pragma once
 
+#include "app/extra_cel.h"
 #include "app/file/format_options.h"
 #include "base/disable_copying.h"
+#include "base/mutex.h"
 #include "base/observable.h"
 #include "base/shared_ptr.h"
 #include "base/unique_ptr.h"
+#include "doc/blend_mode.h"
+#include "doc/color.h"
 #include "doc/document.h"
+#include "doc/frame.h"
+#include "doc/pixel_format.h"
+#include "gfx/rect.h"
 #include "gfx/transformation.h"
-#include "raster/frame_number.h"
-#include "raster/pixel_format.h"
 
 #include <string>
 
-namespace base {
-  class mutex;
-}
-
-namespace raster {
+namespace doc {
   class Cel;
-  class Image;
   class Layer;
   class Mask;
+  class MaskBoundaries;
   class Sprite;
 }
 
@@ -48,16 +38,12 @@ namespace gfx {
   class Region;
 }
 
-namespace undo {
-  class UndoersCollector;
-}
-
 namespace app {
   class DocumentApi;
   class DocumentUndo;
-  struct BoundSeg;
+  class Transaction;
 
-  using namespace raster;
+  using namespace doc;
 
   enum DuplicateType {
     DuplicateExactCopy,
@@ -77,22 +63,26 @@ namespace app {
     ~Document();
 
     // Returns a high-level API: observable and undoable methods.
-    DocumentApi getApi(undo::UndoersCollector* undoers = NULL);
+    DocumentApi getApi(Transaction& transaction);
 
     //////////////////////////////////////////////////////////////////////
     // Main properties
 
-    const DocumentUndo* getUndo() const { return m_undo; }
-    DocumentUndo* getUndo() { return m_undo; }
+    const DocumentUndo* undoHistory() const { return m_undo; }
+    DocumentUndo* undoHistory() { return m_undo; }
+
+    color_t bgColor() const;
+    color_t bgColor(Layer* layer) const;
 
     //////////////////////////////////////////////////////////////////////
     // Notifications
 
     void notifyGeneralUpdate();
-    void notifySpritePixelsModified(Sprite* sprite, const gfx::Region& region);
+    void notifySpritePixelsModified(Sprite* sprite, const gfx::Region& region, frame_t frame);
+    void notifyExposeSpritePixels(Sprite* sprite, const gfx::Region& region);
     void notifyLayerMergedDown(Layer* srcLayer, Layer* targetLayer);
-    void notifyCelMoved(Layer* fromLayer, FrameNumber fromFrame, Layer* toLayer, FrameNumber toFrame);
-    void notifyCelCopied(Layer* fromLayer, FrameNumber fromFrame, Layer* toLayer, FrameNumber toFrame);
+    void notifyCelMoved(Layer* fromLayer, frame_t fromFrame, Layer* toLayer, frame_t toFrame);
+    void notifyCelCopied(Layer* fromLayer, frame_t fromFrame, Layer* toLayer, frame_t toFrame);
     void notifySelectionChanged();
 
     //////////////////////////////////////////////////////////////////////
@@ -108,30 +98,31 @@ namespace app {
     // back to the saved state using the UndoHistory.
     void impossibleToBackToSavedState();
 
+    // Returns true if it does make sense to create a backup in this
+    // document. For example, it doesn't make sense to create a backup
+    // for an unmodified document.
+    bool needsBackup() const;
+
     //////////////////////////////////////////////////////////////////////
     // Loaded options from file
 
-    void setFormatOptions(const SharedPtr<FormatOptions>& format_options);
-    SharedPtr<FormatOptions> getFormatOptions() { return m_format_options; }
+    void setFormatOptions(const base::SharedPtr<FormatOptions>& format_options);
+    base::SharedPtr<FormatOptions> getFormatOptions() { return m_format_options; }
 
     //////////////////////////////////////////////////////////////////////
     // Boundaries
 
-    int getBoundariesSegmentsCount() const;
-    const BoundSeg* getBoundariesSegments() const;
+    void generateMaskBoundaries(const Mask* mask = nullptr);
 
-    void generateMaskBoundaries(Mask* mask = NULL);
+    const MaskBoundaries* getMaskBoundaries() const {
+     return m_maskBoundaries.get();
+    }
 
     //////////////////////////////////////////////////////////////////////
     // Extra Cel (it is used to draw pen preview, pixels in movement, etc.)
 
-    void prepareExtraCel(int x, int y, int w, int h, int opacity);
-    void destroyExtraCel();
-    Cel* getExtraCel() const;
-    Image* getExtraCelImage() const;
-
-    int getExtraCelBlendMode() const { return m_extraCelBlendMode; }
-    void setExtraCelBlendMode(int mode) { m_extraCelBlendMode = mode; }
+    ExtraCelRef extraCel() const { return m_extraCel; }
+    void setExtraCel(const ExtraCelRef& extraCel) { m_extraCel = extraCel; }
 
     //////////////////////////////////////////////////////////////////////
     // Mask
@@ -173,11 +164,11 @@ namespace app {
 
     // Locks the sprite to read or write on it, returning true if the
     // sprite can be accessed in the desired mode.
-    bool lock(LockType lockType);
+    bool lock(LockType lockType, int timeout);
 
     // If you've locked the sprite to read, using this method you can
     // raise your access level to write it.
-    bool lockToWrite();
+    bool lockToWrite(int timeout);
 
     // If you've locked the sprite to write, using this method you can
     // your access level to only read it.
@@ -196,13 +187,10 @@ namespace app {
     bool m_associated_to_file;
 
     // Selected mask region boundaries
-    struct {
-      int nseg;
-      BoundSeg* seg;
-    } m_bound;
+    base::UniquePtr<doc::MaskBoundaries> m_maskBoundaries;
 
     // Mutex to modify the 'locked' flag.
-    base::mutex* m_mutex;
+    base::mutex m_mutex;
 
     // True if some thread is writing the sprite.
     bool m_write_lock;
@@ -211,14 +199,10 @@ namespace app {
     int m_read_locks;
 
     // Data to save the file in the same format that it was loaded
-    SharedPtr<FormatOptions> m_format_options;
+    base::SharedPtr<FormatOptions> m_format_options;
 
     // Extra cel used to draw extra stuff (e.g. editor's pen preview, pixels in movement, etc.)
-    Cel* m_extraCel;
-
-    // Image of the extra cel.
-    Image* m_extraImage;
-    int m_extraCelBlendMode;
+    ExtraCelRef m_extraCel;
 
     // Current mask.
     base::UniquePtr<Mask> m_mask;
