@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2014  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2015  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,9 +15,9 @@
 #include "app/context_access.h"
 #include "app/document_api.h"
 #include "app/document_range.h"
-#include "app/undo_transaction.h"
-#include "raster/layer.h"
-#include "raster/sprite.h"
+#include "app/transaction.h"
+#include "doc/layer.h"
+#include "doc/sprite.h"
 
 #include <stdexcept>
 
@@ -40,6 +29,12 @@ static DocumentRange drop_range_op(
   Document* doc, Op op, const DocumentRange& from,
   DocumentRangePlace place, const DocumentRange& to)
 {
+  if (place != kDocumentRangeBefore &&
+      place != kDocumentRangeAfter) {
+    ASSERT(false);
+    throw std::invalid_argument("Invalid 'place' argument");
+  }
+
   Sprite* sprite = doc->sprite();
 
   // Check noop/trivial/do nothing cases, i.e., move a range to the same place.
@@ -68,13 +63,13 @@ static DocumentRange drop_range_op(
         for (LayerIndex i = from.layerBegin(); i <= from.layerEnd(); ++i)
           if (sprite->indexToLayer(i)->isBackground())
             throw std::runtime_error("The background layer cannot be moved");
+      }
 
-        // Before background
-        if (place == kDocumentRangeBefore) {
-          Layer* background = sprite->indexToLayer(to.layerBegin());
-          if (background && background->isBackground())
-            throw std::runtime_error("You cannot move something below the background layer");
-        }
+      // Before background
+      if (place == kDocumentRangeBefore) {
+        Layer* background = sprite->indexToLayer(to.layerBegin());
+        if (background && background->isBackground())
+          throw std::runtime_error("You cannot move or copy something below the background layer");
       }
       break;
   }
@@ -83,15 +78,18 @@ static DocumentRange drop_range_op(
   switch (op) {
     case Move: undoLabel = "Move Range"; break;
     case Copy: undoLabel = "Copy Range"; break;
+    default:
+      ASSERT(false);
+      throw std::invalid_argument("Invalid 'op' argument");
   }
   DocumentRange resultRange;
 
   {
     const app::Context* context = static_cast<app::Context*>(doc->context());
     const ContextReader reader(context);
-    ContextWriter writer(reader);
-    UndoTransaction undo(writer.context(), undoLabel, undo::ModifyDocument);
-    DocumentApi api = doc->getApi();
+    ContextWriter writer(reader, 500);
+    Transaction transaction(writer.context(), undoLabel, ModifyDocument);
+    DocumentApi api = doc->getApi(transaction);
 
     // TODO Try to add the range with just one call to DocumentApi
     // methods, to avoid generating a lot of SetCelFrame undoers (see
@@ -106,8 +104,8 @@ static DocumentRange drop_range_op(
 
           int srcLayerBegin, srcLayerStep, srcLayerEnd;
           int dstLayerBegin, dstLayerStep;
-          FrameNumber srcFrameBegin, srcFrameStep, srcFrameEnd;
-          FrameNumber dstFrameBegin, dstFrameStep;
+          frame_t srcFrameBegin, srcFrameStep, srcFrameEnd;
+          frame_t dstFrameBegin, dstFrameStep;
 
           if (to.layerBegin() <= from.layerBegin()) {
             srcLayerBegin = from.layerBegin();
@@ -126,23 +124,27 @@ static DocumentRange drop_range_op(
 
           if (to.frameBegin() <= from.frameBegin()) {
             srcFrameBegin = from.frameBegin();
-            srcFrameStep = FrameNumber(1);
-            srcFrameEnd = from.frameEnd().next();
+            srcFrameStep = frame_t(1);
+            srcFrameEnd = from.frameEnd()+1;
             dstFrameBegin = to.frameBegin();
-            dstFrameStep = FrameNumber(1);
+            dstFrameStep = frame_t(1);
           }
           else {
             srcFrameBegin = from.frameEnd();
-            srcFrameStep = FrameNumber(-1);
-            srcFrameEnd = from.frameBegin().previous();
+            srcFrameStep = frame_t(-1);
+            srcFrameEnd = from.frameBegin()-1;
             dstFrameBegin = to.frameEnd();
-            dstFrameStep = FrameNumber(-1);
+            dstFrameStep = frame_t(-1);
           }
 
           for (int srcLayerIdx = srcLayerBegin,
                  dstLayerIdx = dstLayerBegin; srcLayerIdx != srcLayerEnd; ) {
-            for (FrameNumber srcFrame = srcFrameBegin,
+            for (frame_t srcFrame = srcFrameBegin,
                    dstFrame = dstFrameBegin; srcFrame != srcFrameEnd; ) {
+              if (dstLayerIdx < 0 || dstLayerIdx >= int(layers.size()) ||
+                  srcLayerIdx < 0 || srcLayerIdx >= int(layers.size()))
+                break;
+
               LayerImage* srcLayer = static_cast<LayerImage*>(layers[srcLayerIdx]);
               LayerImage* dstLayer = static_cast<LayerImage*>(layers[dstLayerIdx]);
 
@@ -164,8 +166,8 @@ static DocumentRange drop_range_op(
 
       case DocumentRange::kFrames:
         {
-          FrameNumber srcFrameBegin, srcFrameStep, srcFrameEnd;
-          FrameNumber dstFrameBegin, dstFrameStep;
+          frame_t srcFrameBegin = 0, srcFrameStep, srcFrameEnd = 0;
+          frame_t dstFrameBegin = 0, dstFrameStep;
 
           switch (op) {
 
@@ -173,33 +175,33 @@ static DocumentRange drop_range_op(
               if (place == kDocumentRangeBefore) {
                 if (to.frameBegin() <= from.frameBegin()) {
                   srcFrameBegin = from.frameBegin();
-                  srcFrameStep = FrameNumber(1);
-                  srcFrameEnd = from.frameEnd().next();
+                  srcFrameStep = frame_t(1);
+                  srcFrameEnd = from.frameEnd()+1;
                   dstFrameBegin = to.frameBegin();
-                  dstFrameStep = FrameNumber(1);
+                  dstFrameStep = frame_t(1);
                 }
                 else {
                   srcFrameBegin = from.frameEnd();
-                  srcFrameStep = FrameNumber(-1);
-                  srcFrameEnd = from.frameBegin().previous();
+                  srcFrameStep = frame_t(-1);
+                  srcFrameEnd = from.frameBegin()-1;
                   dstFrameBegin = to.frameBegin();
-                  dstFrameStep = FrameNumber(-1);
+                  dstFrameStep = frame_t(-1);
                 }
               }
               else if (place == kDocumentRangeAfter) {
                 if (to.frameEnd() <= from.frameBegin()) {
                   srcFrameBegin = from.frameBegin();
-                  srcFrameStep = FrameNumber(1);
-                  srcFrameEnd = from.frameEnd().next();
-                  dstFrameBegin = to.frameEnd();
-                  dstFrameStep = FrameNumber(1);
+                  srcFrameStep = frame_t(1);
+                  srcFrameEnd = from.frameEnd()+1;
+                  dstFrameBegin = to.frameEnd()+1;
+                  dstFrameStep = frame_t(1);
                 }
                 else {
                   srcFrameBegin = from.frameEnd();
-                  srcFrameStep = FrameNumber(-1);
-                  srcFrameEnd = from.frameBegin().previous();
-                  dstFrameBegin = to.frameEnd().next();
-                  dstFrameStep = FrameNumber(-1);
+                  srcFrameStep = frame_t(-1);
+                  srcFrameEnd = from.frameBegin()-1;
+                  dstFrameBegin = to.frameEnd()+1;
+                  dstFrameStep = frame_t(-1);
                 }
               }
               break;
@@ -208,39 +210,39 @@ static DocumentRange drop_range_op(
               if (place == kDocumentRangeBefore) {
                 if (to.frameBegin() <= from.frameBegin()) {
                   srcFrameBegin = from.frameBegin();
-                  srcFrameStep = FrameNumber(2);
-                  srcFrameEnd = from.frameBegin().next(2*from.frames());
+                  srcFrameStep = frame_t(2);
+                  srcFrameEnd = from.frameBegin() + 2*from.frames();
                   dstFrameBegin = to.frameBegin();
-                  dstFrameStep = FrameNumber(1);
+                  dstFrameStep = frame_t(1);
                 }
                 else {
                   srcFrameBegin = from.frameEnd();
-                  srcFrameStep = FrameNumber(-1);
-                  srcFrameEnd = from.frameBegin().previous();
+                  srcFrameStep = frame_t(-1);
+                  srcFrameEnd = from.frameBegin()-1;
                   dstFrameBegin = to.frameBegin();
-                  dstFrameStep = FrameNumber(0);
+                  dstFrameStep = frame_t(0);
                 }
               }
               else if (place == kDocumentRangeAfter) {
                 if (to.frameEnd() <= from.frameBegin()) {
                   srcFrameBegin = from.frameBegin();
-                  srcFrameStep = FrameNumber(2);
-                  srcFrameEnd = from.frameBegin().next(2*from.frames());
-                  dstFrameBegin = to.frameEnd().next();
-                  dstFrameStep = FrameNumber(1);
+                  srcFrameStep = frame_t(2);
+                  srcFrameEnd = from.frameBegin() + 2*from.frames();
+                  dstFrameBegin = to.frameEnd()+1;
+                  dstFrameStep = frame_t(1);
                 }
                 else {
                   srcFrameBegin = from.frameEnd();
-                  srcFrameStep = FrameNumber(-1);
-                  srcFrameEnd = from.frameBegin().previous();
-                  dstFrameBegin = to.frameEnd().next();
-                  dstFrameStep = FrameNumber(0);
+                  srcFrameStep = frame_t(-1);
+                  srcFrameEnd = from.frameBegin()-1;
+                  dstFrameBegin = to.frameEnd()+1;
+                  dstFrameStep = frame_t(0);
                 }
               }
               break;
           }
 
-          for (FrameNumber srcFrame = srcFrameBegin,
+          for (frame_t srcFrame = srcFrameBegin,
                  dstFrame = dstFrameBegin; srcFrame != srcFrameEnd; ) {
             switch (op) {
               case Move: api.moveFrame(sprite, srcFrame, dstFrame); break;
@@ -251,12 +253,12 @@ static DocumentRange drop_range_op(
           }
 
           if (place == kDocumentRangeBefore) {
-            resultRange.startRange(LayerIndex::NoLayer, FrameNumber(to.frameBegin()), from.type());
-            resultRange.endRange(LayerIndex::NoLayer, FrameNumber(to.frameBegin()+from.frames()-1));
+            resultRange.startRange(LayerIndex::NoLayer, frame_t(to.frameBegin()), from.type());
+            resultRange.endRange(LayerIndex::NoLayer, frame_t(to.frameBegin()+from.frames()-1));
           }
           else if (place == kDocumentRangeAfter) {
-            resultRange.startRange(LayerIndex::NoLayer, FrameNumber(to.frameEnd()+1), from.type());
-            resultRange.endRange(LayerIndex::NoLayer, FrameNumber(to.frameEnd()+1+from.frames()-1));
+            resultRange.startRange(LayerIndex::NoLayer, frame_t(to.frameEnd()+1), from.type());
+            resultRange.endRange(LayerIndex::NoLayer, frame_t(to.frameEnd()+1+from.frames()-1));
           }
 
           if (op == Move && from.frameBegin() < to.frameBegin())
@@ -310,12 +312,12 @@ static DocumentRange drop_range_op(
           }
 
           if (place == kDocumentRangeBefore) {
-            resultRange.startRange(LayerIndex(to.layerBegin()), FrameNumber(-1), from.type());
-            resultRange.endRange(LayerIndex(to.layerBegin()+from.layers()-1), FrameNumber(-1));
+            resultRange.startRange(LayerIndex(to.layerBegin()), frame_t(-1), from.type());
+            resultRange.endRange(LayerIndex(to.layerBegin()+from.layers()-1), frame_t(-1));
           }
           else if (place == kDocumentRangeAfter) {
-            resultRange.startRange(LayerIndex(to.layerEnd()+1), FrameNumber(-1), from.type());
-            resultRange.endRange(LayerIndex(to.layerEnd()+1+from.layers()-1), FrameNumber(-1));
+            resultRange.startRange(LayerIndex(to.layerEnd()+1), frame_t(-1), from.type());
+            resultRange.endRange(LayerIndex(to.layerEnd()+1+from.layers()-1), frame_t(-1));
           }
 
           if (op == Move && from.layerBegin() < to.layerBegin())
@@ -324,7 +326,7 @@ static DocumentRange drop_range_op(
         break;
     }
 
-    undo.commit();
+    transaction.commit();
   }
 
   return resultRange;
@@ -344,11 +346,11 @@ void reverse_frames(Document* doc, const DocumentRange& range)
 {
   const app::Context* context = static_cast<app::Context*>(doc->context());
   const ContextReader reader(context);
-  ContextWriter writer(reader);
-  UndoTransaction undo(writer.context(), "Reverse Frames");
-  DocumentApi api = doc->getApi();
+  ContextWriter writer(reader, 500);
+  Transaction transaction(writer.context(), "Reverse Frames");
+  DocumentApi api = doc->getApi(transaction);
   Sprite* sprite = doc->sprite();
-  FrameNumber frameBegin, frameEnd;
+  frame_t frameBegin, frameEnd;
   int layerBegin, layerEnd;
   bool moveFrames = false;
 
@@ -365,17 +367,17 @@ void reverse_frames(Document* doc, const DocumentRange& range)
       moveFrames = true;
       break;
     case DocumentRange::kLayers:
-      frameBegin = FrameNumber(0);
-      frameEnd = sprite->totalFrames().previous();
+      frameBegin = frame_t(0);
+      frameEnd = sprite->totalFrames()-1;
       layerBegin = range.layerBegin();
       layerEnd = range.layerEnd() + 1;
       break;
   }
 
   if (moveFrames) {
-    for (FrameNumber frameRev = frameEnd.next();
+    for (frame_t frameRev = frameEnd+1;
          frameRev > frameBegin;
-         frameRev = frameRev.previous()) {
+         --frameRev) {
       api.moveFrame(sprite, frameBegin, frameRev);
     }
   }
@@ -384,18 +386,17 @@ void reverse_frames(Document* doc, const DocumentRange& range)
     sprite->getLayersList(layers);
 
     for (int layerIdx = layerBegin; layerIdx != layerEnd; ++layerIdx) {
-      for (FrameNumber frame = frameBegin,
+      for (frame_t frame = frameBegin,
              frameRev = frameEnd;
-           frame != FrameNumber((frameBegin+frameEnd)/2).next();
-           frame = frame.next(),
-             frameRev = frameRev.previous()) {
+           frame != (frameBegin+frameEnd)/2+1;
+           ++frame, --frameRev) {
         LayerImage* layer = static_cast<LayerImage*>(layers[layerIdx]);
         api.swapCel(layer, frame, frameRev);
       }
     }
   }
 
-  undo.commit();
+  transaction.commit();
 }
 
 } // namespace app
