@@ -1,35 +1,25 @@
-/* Aseprite
- * Copyright (C) 2001-2014  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2015  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #include "tests/test.h"
 
+#include "app/context.h"
 #include "app/document.h"
 #include "app/document_api.h"
 #include "app/document_range.h"
 #include "app/document_range_ops.h"
 #include "app/document_undo.h"
-#include "app/test_context.h"
+#include "app/transaction.h"
 #include "base/unique_ptr.h"
-#include "raster/raster.h"
-#include "undo/undo_history.h"
+#include "doc/doc.h"
+#include "doc/test_context.h"
 
 using namespace app;
-using namespace raster;
+using namespace doc;
 
 namespace app {
 
@@ -40,7 +30,7 @@ namespace app {
 
 }
 
-typedef base::UniquePtr<Document> DocumentPtr;
+typedef base::UniquePtr<app::Document> DocumentPtr;
 
 #define EXPECT_LAYER_ORDER(a, b, c, d) \
   EXPECT_TRUE(expect_layer(a, 0));     \
@@ -90,7 +80,7 @@ public:
     black = rgba(0, 0, 0, 0);
     white = rgba(255, 255, 255, 255);
 
-    doc.reset(static_cast<Document*>(ctx.documents().add(4, 4)));
+    doc.reset(static_cast<app::Document*>(ctx.documents().add(4, 4)));
     sprite = doc->sprite();
     layer1 = dynamic_cast<LayerImage*>(sprite->folder()->getFirstLayer());
     layer2 = new LayerImage(sprite);
@@ -106,30 +96,28 @@ public:
     layer3->setName("layer3");
     layer4->setName("layer4");
 
-    sprite->setTotalFrames(FrameNumber(4));
-    sprite->setFrameDuration(FrameNumber(0), 1); // These durations can be used to identify
-    sprite->setFrameDuration(FrameNumber(1), 2); // frames after a move operation
-    sprite->setFrameDuration(FrameNumber(2), 3);
-    sprite->setFrameDuration(FrameNumber(3), 4);
+    sprite->setTotalFrames(frame_t(4));
+    sprite->setFrameDuration(frame_t(0), 1); // These durations can be used to identify
+    sprite->setFrameDuration(frame_t(1), 2); // frames after a move operation
+    sprite->setFrameDuration(frame_t(2), 3);
+    sprite->setFrameDuration(frame_t(3), 4);
 
-    DocumentApi api = doc->getApi();
     for (int i=0; i<4; i++) {
       LayerImage* layer = static_cast<LayerImage*>(sprite->indexToLayer(LayerIndex(i)));
 
       for (int j=0; j<4; j++) {
-        Cel* cel = layer->getCel(FrameNumber(j));
-        Image* image;
+        Cel* cel = layer->cel(frame_t(j));
+        ImageRef image;
         if (cel)
-          image = cel->image();
+          image = cel->imageRef();
         else {
-          image = Image::create(IMAGE_RGB, 4, 4);
-          int imageIdx = sprite->stock()->addImage(image);
-          cel = new Cel(FrameNumber(j), imageIdx);
+          image.reset(Image::create(IMAGE_RGB, 4, 4));
+          cel = new Cel(frame_t(j), image);
           layer->addCel(cel);
         }
 
-        clear_image(image, black);
-        put_pixel(image, i, j, white);
+        clear_image(image.get(), black);
+        put_pixel(image.get(), i, j, white);
       }
     }
   }
@@ -143,7 +131,7 @@ protected:
     return expect_layer_frame(sprite->layerToIndex(expected_layer), -1, layer, -1);
   }
 
-  bool expect_frame(int expected_frame, int frame) {
+  bool expect_frame(int expected_frame, frame_t frame) {
     for (int i=0; i<(int)sprite->countLayers(); ++i) {
       if (!expect_layer_frame(i, expected_frame, i, frame))
         return false;
@@ -151,13 +139,13 @@ protected:
     return true;
   }
 
-  bool expect_layer_frame(int expected_layer, int expected_frame, int layer, int frame) {
+  bool expect_layer_frame(int expected_layer, int expected_frame, int layer, frame_t frame) {
     if (frame >= 0) {
       if (!expect_cel(expected_layer, expected_frame, layer, frame))
         return false;
 
-      EXPECT_EQ((expected_frame+1), sprite->getFrameDuration(FrameNumber(frame)));
-      return ((expected_frame+1) == sprite->getFrameDuration(FrameNumber(frame)));
+      EXPECT_EQ((expected_frame+1), sprite->frameDuration(frame));
+      return ((expected_frame+1) == sprite->frameDuration(frame));
     }
 
     if (layer >= 0) {
@@ -171,12 +159,15 @@ protected:
     return true;
   }
 
-  bool expect_cel(int expected_layer, int expected_frame, int layer, int frame) {
+  bool expect_cel(int expected_layer, int expected_frame, int layer, frame_t frame) {
     color_t expected_color = white;
 
+    Cel* cel = sprite->indexToLayer(LayerIndex(layer))->cel(frame);
+    if (!cel)
+      return false;
+
     color_t color = get_pixel(
-      static_cast<LayerImage*>(sprite->indexToLayer(LayerIndex(layer)))
-      ->getCel(FrameNumber(frame))->image(),
+      cel->image(),
       expected_layer, expected_frame);
 
     EXPECT_EQ(expected_color, color);
@@ -184,15 +175,14 @@ protected:
     return (expected_color == color);
   }
 
-  bool expect_empty_cel(int layer, int frame) {
-    Cel* cel = static_cast<LayerImage*>(sprite->indexToLayer(LayerIndex(layer)))
-      ->getCel(FrameNumber(frame));
+  bool expect_empty_cel(int layer, frame_t frame) {
+    Cel* cel = sprite->indexToLayer(LayerIndex(layer))->cel(frame);
 
     EXPECT_EQ(NULL, cel);
     return (cel == NULL);
   }
 
-  TestContext ctx;
+  TestContextT<app::Context> ctx;
   DocumentPtr doc;
   Sprite* sprite;
   LayerImage* layer1;
@@ -203,17 +193,17 @@ protected:
   color_t white;
 };
 
-inline DocumentRange range(Layer* fromLayer, int fromFrNum, Layer* toLayer, int toFrNum, DocumentRange::Type type) {
+inline DocumentRange range(Layer* fromLayer, frame_t fromFrNum, Layer* toLayer, frame_t toFrNum, DocumentRange::Type type) {
   DocumentRange r;
-  r.startRange(fromLayer->sprite()->layerToIndex(fromLayer), FrameNumber(fromFrNum), type);
-  r.endRange(toLayer->sprite()->layerToIndex(toLayer), FrameNumber(toFrNum));
+  r.startRange(fromLayer->sprite()->layerToIndex(fromLayer), fromFrNum, type);
+  r.endRange(toLayer->sprite()->layerToIndex(toLayer), toFrNum);
   return r;
 }
 
-inline DocumentRange range(int fromLayer, int fromFrNum, int toLayer, int toFrNum, DocumentRange::Type type) {
+inline DocumentRange range(int fromLayer, frame_t fromFrNum, int toLayer, frame_t toFrNum, DocumentRange::Type type) {
   DocumentRange r;
-  r.startRange(LayerIndex(fromLayer), FrameNumber(fromFrNum), type);
-  r.endRange(LayerIndex(toLayer), FrameNumber(toFrNum));
+  r.startRange(LayerIndex(fromLayer), fromFrNum, type);
+  r.endRange(LayerIndex(toLayer), toFrNum);
   return r;
 }
 
@@ -233,15 +223,15 @@ inline DocumentRange layers_range(int layer) {
   return range(layer, -1, layer, -1, DocumentRange::kLayers);
 }
 
-inline DocumentRange frames_range(int fromFrame, int toFrame) {
+inline DocumentRange frames_range(frame_t fromFrame, frame_t toFrame) {
   return range(0, fromFrame, 0, toFrame, DocumentRange::kFrames);
 }
 
-inline DocumentRange frames_range(int frame) {
+inline DocumentRange frames_range(frame_t frame) {
   return range(0, frame, 0, frame, DocumentRange::kFrames);
 }
 
-inline DocumentRange cels_range(int fromLayer, int fromFrNum, int toLayer, int toFrNum) {
+inline DocumentRange cels_range(int fromLayer, frame_t fromFrNum, int toLayer, frame_t toFrNum) {
   return range(fromLayer, fromFrNum, toLayer, toFrNum, DocumentRange::kCels);
 }
 
@@ -253,35 +243,35 @@ TEST_F(DocRangeOps, MoveLayersNoOp) {
       layers_range(layer1),
       layers_range(layer1), kDocumentRangeAfter));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer1),
     move_range(doc,
       layers_range(layer1),
       layers_range(layer2), kDocumentRangeBefore));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer4),
     move_range(doc,
       layers_range(layer4),
       layers_range(layer4), kDocumentRangeAfter));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer4),
     move_range(doc,
       layers_range(layer4),
       layers_range(layer4), kDocumentRangeBefore));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer4),
     move_range(doc,
       layers_range(layer4),
       layers_range(layer3), kDocumentRangeAfter));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   // Move two layer to the same place
 
@@ -290,70 +280,70 @@ TEST_F(DocRangeOps, MoveLayersNoOp) {
       layers_range(layer1, layer2),
       layers_range(layer1), kDocumentRangeBefore));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer1, layer2),
     move_range(doc,
       layers_range(layer1, layer2),
       layers_range(layer1), kDocumentRangeAfter));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer1, layer2),
     move_range(doc,
       layers_range(layer1, layer2),
       layers_range(layer2), kDocumentRangeBefore));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer1, layer2),
     move_range(doc,
       layers_range(layer1, layer2),
       layers_range(layer2), kDocumentRangeAfter));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer1, layer2),
     move_range(doc,
       layers_range(layer1, layer2),
       layers_range(layer3), kDocumentRangeBefore));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer3, layer4),
     move_range(doc,
       layers_range(layer3, layer4),
       layers_range(layer2), kDocumentRangeAfter));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer3, layer4),
     move_range(doc,
       layers_range(layer3, layer4),
       layers_range(layer3), kDocumentRangeBefore));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer3, layer4),
     move_range(doc,
       layers_range(layer3, layer4),
       layers_range(layer3), kDocumentRangeAfter));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer3, layer4),
     move_range(doc,
       layers_range(layer3, layer4),
       layers_range(layer4), kDocumentRangeBefore));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(layers_range(layer3, layer4),
     move_range(doc,
       layers_range(layer3, layer4),
       layers_range(layer4), kDocumentRangeAfter));
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   // Move four layers
 
@@ -365,7 +355,7 @@ TEST_F(DocRangeOps, MoveLayersNoOp) {
           layers_range(layer1, layer4),
           layers_range(layer), places[i]));
       EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-      EXPECT_FALSE(doc->getUndo()->canUndo());
+      EXPECT_FALSE(doc->undoHistory()->canUndo());
     }
 
     for (int layer=0; layer<3; ++layer) {
@@ -374,7 +364,7 @@ TEST_F(DocRangeOps, MoveLayersNoOp) {
           layers_range(layer1, layer4),
           layers_range(layer, layer+1), places[i]));
       EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-      EXPECT_FALSE(doc->getUndo()->canUndo());
+      EXPECT_FALSE(doc->undoHistory()->canUndo());
     }
 
     for (int layer=0; layer<2; ++layer) {
@@ -383,7 +373,7 @@ TEST_F(DocRangeOps, MoveLayersNoOp) {
           layers_range(layer1, layer4),
           layers_range(layer, layer+2), places[i]));
       EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-      EXPECT_FALSE(doc->getUndo()->canUndo());
+      EXPECT_FALSE(doc->undoHistory()->canUndo());
     }
 
     EXPECT_EQ(layers_range(layer1, layer4),
@@ -391,7 +381,7 @@ TEST_F(DocRangeOps, MoveLayersNoOp) {
         layers_range(layer1, layer4),
         layers_range(layer1, layer4), places[i]));
     EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
-    EXPECT_FALSE(doc->getUndo()->canUndo());
+    EXPECT_FALSE(doc->undoHistory()->canUndo());
   }
 }
 
@@ -403,35 +393,35 @@ TEST_F(DocRangeOps, MoveFramesNoOp) {
       frames_range(0),
       frames_range(0), kDocumentRangeAfter));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(0),
     move_range(doc,
       frames_range(0),
       frames_range(1), kDocumentRangeBefore));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(3),
     move_range(doc,
       frames_range(3),
       frames_range(3), kDocumentRangeAfter));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(3),
     move_range(doc,
       frames_range(3),
       frames_range(3), kDocumentRangeBefore));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(3),
     move_range(doc,
       frames_range(3),
       frames_range(2), kDocumentRangeAfter));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   // Move two frame to the same place
 
@@ -440,70 +430,70 @@ TEST_F(DocRangeOps, MoveFramesNoOp) {
       frames_range(0, 1),
       frames_range(0), kDocumentRangeBefore));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(0, 1),
     move_range(doc,
       frames_range(0, 1),
       frames_range(0), kDocumentRangeAfter));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(0, 1),
     move_range(doc,
       frames_range(0, 1),
       frames_range(1), kDocumentRangeBefore));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(0, 1),
     move_range(doc,
       frames_range(0, 1),
       frames_range(1), kDocumentRangeAfter));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(0, 1),
     move_range(doc,
       frames_range(0, 1),
       frames_range(2), kDocumentRangeBefore));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(2, 3),
     move_range(doc,
       frames_range(2, 3),
       frames_range(1), kDocumentRangeAfter));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(2, 3),
     move_range(doc,
       frames_range(2, 3),
       frames_range(2), kDocumentRangeBefore));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(2, 3),
     move_range(doc,
       frames_range(2, 3),
       frames_range(2), kDocumentRangeAfter));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(2, 3),
     move_range(doc,
       frames_range(2, 3),
       frames_range(3), kDocumentRangeBefore));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   EXPECT_EQ(frames_range(2, 3),
     move_range(doc,
       frames_range(2, 3),
       frames_range(3), kDocumentRangeAfter));
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
-  EXPECT_FALSE(doc->getUndo()->canUndo());
+  EXPECT_FALSE(doc->undoHistory()->canUndo());
 
   // Move four frames
 
@@ -515,7 +505,7 @@ TEST_F(DocRangeOps, MoveFramesNoOp) {
           frames_range(0, 3),
           frames_range(frame), places[i]));
       EXPECT_FRAME_ORDER(0, 1, 2, 3);
-      EXPECT_FALSE(doc->getUndo()->canUndo());
+      EXPECT_FALSE(doc->undoHistory()->canUndo());
     }
 
     for (int frame=0; frame<3; ++frame) {
@@ -524,7 +514,7 @@ TEST_F(DocRangeOps, MoveFramesNoOp) {
           frames_range(0, 3),
           frames_range(frame, frame+1), places[i]));
       EXPECT_FRAME_ORDER(0, 1, 2, 3);
-      EXPECT_FALSE(doc->getUndo()->canUndo());
+      EXPECT_FALSE(doc->undoHistory()->canUndo());
     }
 
     for (int frame=0; frame<2; ++frame) {
@@ -533,7 +523,7 @@ TEST_F(DocRangeOps, MoveFramesNoOp) {
           frames_range(0, 3),
           frames_range(frame, frame+2), places[i]));
       EXPECT_FRAME_ORDER(0, 1, 2, 3);
-      EXPECT_FALSE(doc->getUndo()->canUndo());
+      EXPECT_FALSE(doc->undoHistory()->canUndo());
     }
 
     EXPECT_EQ(frames_range(0, 3),
@@ -541,7 +531,7 @@ TEST_F(DocRangeOps, MoveFramesNoOp) {
         frames_range(0, 3),
         frames_range(0, 3), places[i]));
     EXPECT_FRAME_ORDER(0, 1, 2, 3);
-    EXPECT_FALSE(doc->getUndo()->canUndo());
+    EXPECT_FALSE(doc->undoHistory()->canUndo());
   }
 }
 
@@ -563,7 +553,7 @@ TEST_F(DocRangeOps, MoveLayers) {
   EXPECT_LAYER_ORDER(layer2, layer1, layer3, layer4);
   EXPECT_EQ(layers_range(layer1), result);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
 
   // One layer at the bottom
@@ -573,7 +563,7 @@ TEST_F(DocRangeOps, MoveLayers) {
   EXPECT_LAYER_ORDER(layer2, layer1, layer3, layer4);
   EXPECT_EQ(layers_range(layer2), result);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
 
   // Try with a background
@@ -593,7 +583,7 @@ TEST_F(DocRangeOps, MoveLayers) {
   EXPECT_LAYER_ORDER(layer1, layer3, layer4, layer2);
   EXPECT_EQ(layers_range(layer2), result);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
 
   // Move one layers before other
@@ -603,7 +593,7 @@ TEST_F(DocRangeOps, MoveLayers) {
   EXPECT_LAYER_ORDER(layer1, layer3, layer2, layer4);
   EXPECT_EQ(layers_range(layer2), result);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
 
   result = move_range(doc,
@@ -612,7 +602,7 @@ TEST_F(DocRangeOps, MoveLayers) {
   EXPECT_LAYER_ORDER(layer2, layer1, layer3, layer4);
   EXPECT_EQ(layers_range(layer1), result);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
 
   // Move two layers on top of other
@@ -645,7 +635,7 @@ TEST_F(DocRangeOps, MoveLayers) {
   EXPECT_LAYER_ORDER(layer4, layer1, layer2, layer3);
   EXPECT_EQ(layers_range(layer1, layer3), result);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
 
   // Move three layers at the bottom
@@ -655,7 +645,7 @@ TEST_F(DocRangeOps, MoveLayers) {
   EXPECT_LAYER_ORDER(layer2, layer3, layer4, layer1);
   EXPECT_EQ(layers_range(layer2, layer4), result);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_LAYER_ORDER(layer1, layer2, layer3, layer4);
 }
 
@@ -665,7 +655,7 @@ TEST_F(DocRangeOps, MoveFrames) {
     frames_range(1, 1), kDocumentRangeAfter);
   EXPECT_FRAME_ORDER(1, 0, 2, 3);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   // Move one frame at the end
@@ -674,7 +664,7 @@ TEST_F(DocRangeOps, MoveFrames) {
     frames_range(3, 3), kDocumentRangeAfter);
   EXPECT_FRAME_ORDER(0, 2, 3, 1);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   // Move two frames after other
@@ -683,12 +673,15 @@ TEST_F(DocRangeOps, MoveFrames) {
     frames_range(3, 3), kDocumentRangeAfter);
   EXPECT_FRAME_ORDER(0, 3, 1, 2);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   move_range(doc,
-    frames_range(1, 2),
+    frames_range(2, 3),
     frames_range(0, 0), kDocumentRangeAfter);
+  EXPECT_FRAME_ORDER(0, 2, 3, 1);
+
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   // Move three frames at the beginning
@@ -697,7 +690,7 @@ TEST_F(DocRangeOps, MoveFrames) {
     frames_range(0, 0), kDocumentRangeBefore);
   EXPECT_FRAME_ORDER(1, 2, 3, 0);
 
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 }
 
@@ -709,7 +702,7 @@ TEST_F(DocRangeOps, MoveCels) {
     cels_range(0, 1, 0, 1), ignore);
   EXPECT_CEL(0, 0, 0, 1);
   EXPECT_EMPTY_CEL(0, 0);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_CEL(0, 0, 0, 0);
 
   move_range(doc,
@@ -719,7 +712,7 @@ TEST_F(DocRangeOps, MoveCels) {
   EXPECT_CEL(0, 1, 0, 3);
   EXPECT_EMPTY_CEL(0, 0);
   EXPECT_EMPTY_CEL(0, 1);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
 
   move_range(doc,
     cels_range(0, 0, 0, 3),
@@ -732,7 +725,7 @@ TEST_F(DocRangeOps, MoveCels) {
   EXPECT_EMPTY_CEL(0, 1);
   EXPECT_EMPTY_CEL(0, 2);
   EXPECT_EMPTY_CEL(0, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
 }
 
 TEST_F(DocRangeOps, CopyLayers) {
@@ -745,28 +738,28 @@ TEST_F(DocRangeOps, CopyFrames) {
     frames_range(0),
     frames_range(2, 3), kDocumentRangeBefore);
   EXPECT_FRAME_COPY1(0, 1, 0, 2, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   copy_range(doc,
     frames_range(0),
     frames_range(2, 3), kDocumentRangeAfter);
   EXPECT_FRAME_COPY1(0, 1, 2, 3, 0);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   copy_range(doc,
     frames_range(3),
     frames_range(0, 1), kDocumentRangeBefore);
   EXPECT_FRAME_COPY1(3, 0, 1, 2, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   copy_range(doc,
     frames_range(3),
     frames_range(0, 1), kDocumentRangeAfter);
   EXPECT_FRAME_COPY1(0, 1, 3, 2, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   // Copy three frames
@@ -775,42 +768,42 @@ TEST_F(DocRangeOps, CopyFrames) {
     frames_range(0, 2),
     frames_range(3), kDocumentRangeBefore);
   EXPECT_FRAME_COPY3(0, 1, 2, 0, 1, 2, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   copy_range(doc,
     frames_range(0, 2),
     frames_range(3), kDocumentRangeAfter);
   EXPECT_FRAME_COPY3(0, 1, 2, 3, 0, 1, 2);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   copy_range(doc,
     frames_range(1, 3),
     frames_range(0), kDocumentRangeBefore);
   EXPECT_FRAME_COPY3(1, 2, 3, 0, 1, 2, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   copy_range(doc,
     frames_range(1, 3),
     frames_range(0), kDocumentRangeAfter);
   EXPECT_FRAME_COPY3(0, 1, 2, 3, 1, 2, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   copy_range(doc,
     frames_range(0, 2),
     frames_range(0, 2), kDocumentRangeBefore);
   EXPECT_FRAME_COPY3(0, 1, 2, 0, 1, 2, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   copy_range(doc,
     frames_range(0, 2),
     frames_range(0, 2), kDocumentRangeAfter);
   EXPECT_FRAME_COPY3(0, 1, 2, 0, 1, 2, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 }
 
@@ -827,22 +820,22 @@ TEST_F(DocRangeOps, ReverseFrames) {
 
   reverse_frames(doc, frames_range(1, 2));
   EXPECT_FRAME_ORDER(0, 2, 1, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   reverse_frames(doc, frames_range(0, 2));
   EXPECT_FRAME_ORDER(2, 1, 0, 3);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   reverse_frames(doc, frames_range(1, 3));
   EXPECT_FRAME_ORDER(0, 3, 2, 1);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 
   reverse_frames(doc, frames_range(0, 3));
   EXPECT_FRAME_ORDER(3, 2, 1, 0);
-  doc->getUndo()->doUndo();
+  doc->undoHistory()->undo();
   EXPECT_FRAME_ORDER(0, 1, 2, 3);
 }
 
