@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,7 +16,7 @@
 #include "app/file/format_options.h"
 #include "app/ini_file.h"
 #include "base/file_handle.h"
-#include "raster/raster.h"
+#include "doc/doc.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,9 +28,9 @@ namespace app {
 using namespace base;
 
 class PngFormat : public FileFormat {
-  const char* onGetName() const { return "png"; }
-  const char* onGetExtensions() const { return "png"; }
-  int onGetFlags() const {
+  const char* onGetName() const override { return "png"; }
+  const char* onGetExtensions() const override { return "png"; }
+  int onGetFlags() const override {
     return
       FILE_SUPPORT_LOAD |
       FILE_SUPPORT_SAVE |
@@ -50,7 +39,8 @@ class PngFormat : public FileFormat {
       FILE_SUPPORT_GRAY |
       FILE_SUPPORT_GRAYA |
       FILE_SUPPORT_INDEXED |
-      FILE_SUPPORT_SEQUENCES;
+      FILE_SUPPORT_SEQUENCES |
+      FILE_SUPPORT_PALETTE_WITH_ALPHA;
   }
 
   bool onLoad(FileOp* fop) override;
@@ -66,7 +56,7 @@ FileFormat* CreatePngFormat()
 
 static void report_png_error(png_structp png_ptr, png_const_charp error)
 {
-  fop_error((FileOp*)png_get_error_ptr(png_ptr), "libpng: %s\n", error);
+  ((FileOp*)png_get_error_ptr(png_ptr))->setError("libpng: %s\n", error);
 }
 
 bool PngFormat::onLoad(FileOp* fop)
@@ -79,10 +69,11 @@ bool PngFormat::onLoad(FileOp* fop)
   int pass, number_passes;
   int num_palette;
   png_colorp palette;
-  png_bytep row_pointer;
+  png_bytepp rows_pointer;
   PixelFormat pixelFormat;
 
-  FileHandle fp(open_file_with_exception(fop->filename, "rb"));
+  FileHandle handle(open_file_with_exception(fop->filename(), "rb"));
+  FILE* fp = handle.get();
 
   /* Create and initialize the png_struct with the desired error handler
    * functions.  If you want to use the default stderr and longjump method,
@@ -93,14 +84,14 @@ bool PngFormat::onLoad(FileOp* fop)
   png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, (png_voidp)fop,
                                    report_png_error, report_png_error);
   if (png_ptr == NULL) {
-    fop_error(fop, "png_create_read_struct\n");
+    fop->setError("png_create_read_struct\n");
     return false;
   }
 
   /* Allocate/initialize the memory for image information. */
   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == NULL) {
-    fop_error(fop, "png_create_info_struct\n");
+    fop->setError("png_create_info_struct\n");
     png_destroy_read_struct(&png_ptr, NULL, NULL);
     return false;
   }
@@ -109,7 +100,7 @@ bool PngFormat::onLoad(FileOp* fop)
    * the normal method of doing things with libpng).
    */
   if (setjmp(png_jmpbuf(png_ptr))) {
-    fop_error(fop, "Error reading PNG file\n");
+    fop->setError("Error reading PNG file\n");
     /* Free all of the memory associated with the png_ptr and info_ptr */
     png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
     /* If we get here, we had a problem reading the file */
@@ -164,13 +155,13 @@ bool PngFormat::onLoad(FileOp* fop)
   switch (png_get_color_type(png_ptr, info_ptr)) {
 
     case PNG_COLOR_TYPE_RGB_ALPHA:
-      fop->seq.has_alpha = true;
+      fop->sequenceSetHasAlpha(true);
     case PNG_COLOR_TYPE_RGB:
       pixelFormat = IMAGE_RGB;
       break;
 
     case PNG_COLOR_TYPE_GRAY_ALPHA:
-      fop->seq.has_alpha = true;
+      fop->sequenceSetHasAlpha(true);
     case PNG_COLOR_TYPE_GRAY:
       pixelFormat = IMAGE_GRAYSCALE;
       break;
@@ -180,151 +171,169 @@ bool PngFormat::onLoad(FileOp* fop)
       break;
 
     default:
-      fop_error(fop, "Color type not supported\n)");
+      fop->setError("Color type not supported\n)");
       png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
       return false;
   }
 
   int imageWidth = png_get_image_width(png_ptr, info_ptr);
   int imageHeight = png_get_image_height(png_ptr, info_ptr);
-  Image* image = fop_sequence_image(fop, pixelFormat, imageWidth, imageHeight);
+  Image* image = fop->sequenceImage(pixelFormat, imageWidth, imageHeight);
   if (!image) {
-    fop_error(fop, "file_sequence_image %dx%d\n", imageWidth, imageHeight);
+    fop->setError("file_sequence_image %dx%d\n", imageWidth, imageHeight);
     png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
     return false;
   }
 
-  // Transparent palette entries
-  std::vector<uint8_t> pal_alphas(256, 255);
-  int mask_entry = -1;
+  // Transparent color
+  png_color_16p png_trans_color = NULL;
 
   // Read the palette
   if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE &&
       png_get_PLTE(png_ptr, info_ptr, &palette, &num_palette)) {
-    int c;
+    fop->sequenceSetNColors(num_palette);
 
-    for (c = 0; c < num_palette; c++) {
-      fop_sequence_set_color(fop, c,
-                             palette[c].red,
-                             palette[c].green,
-                             palette[c].blue);
-    }
-    for (; c < 256; c++) {
-      fop_sequence_set_color(fop, c, 0, 0, 0);
+    for (int c=0; c<num_palette; ++c) {
+      fop->sequenceSetColor(c,
+                            palette[c].red,
+                            palette[c].green,
+                            palette[c].blue);
     }
 
     // Read alpha values for palette entries
     png_bytep trans = NULL;     // Transparent palette entries
     int num_trans = 0;
+    int mask_entry = -1;
 
-    png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans, NULL);
+    png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans, nullptr);
 
     for (int i = 0; i < num_trans; ++i) {
-      pal_alphas[i] = trans[i];
+      fop->sequenceSetAlpha(i, trans[i]);
 
-      if (pal_alphas[i] < 128) {
-        fop->seq.has_alpha = true; // Is a transparent sprite
-
-        if (mask_entry < 0)
-          mask_entry = i;
+      if (trans[i] < 255) {
+        fop->sequenceSetHasAlpha(true); // Is a transparent sprite
+        if (trans[i] == 0) {
+          if (mask_entry < 0)
+            mask_entry = i;
+        }
       }
     }
 
-    // Set the transparent color to the first transparent entry found
     if (mask_entry >= 0)
-      fop->document->sprite()->setTransparentColor(mask_entry);
+      fop->document()->sprite()->setTransparentColor(mask_entry);
+  }
+  else {
+    png_get_tRNS(png_ptr, info_ptr, nullptr, nullptr, &png_trans_color);
   }
 
-  mask_entry = fop->document->sprite()->transparentColor();
+  // Allocate the memory to hold the image using the fields of info_ptr.
+  rows_pointer = (png_bytepp)png_malloc(png_ptr, sizeof(png_bytep) * height);
+  for (y = 0; y < height; y++)
+    rows_pointer[y] = (png_bytep)png_malloc(png_ptr, png_get_rowbytes(png_ptr, info_ptr));
 
-  /* Allocate the memory to hold the image using the fields of info_ptr. */
-
-   /* The easiest way to read the image: */
-  row_pointer = (png_bytep)png_malloc(png_ptr, png_get_rowbytes(png_ptr, info_ptr));
   for (pass = 0; pass < number_passes; pass++) {
     for (y = 0; y < height; y++) {
-      /* read the line */
-      png_read_row(png_ptr, row_pointer, (png_byte*)NULL);
+      png_read_rows(png_ptr, rows_pointer+y, nullptr, 1);
 
-      /* RGB_ALPHA */
-      if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB_ALPHA) {
-        uint8_t* src_address = row_pointer;
-        uint32_t* dst_address = (uint32_t*)image->getPixelAddress(0, y);
-        unsigned int x, r, g, b, a;
+      fop->setProgress(
+        (double)((double)pass + (double)(y+1) / (double)(height))
+        / (double)number_passes);
 
-        for (x=0; x<width; x++) {
-          r = *(src_address++);
-          g = *(src_address++);
-          b = *(src_address++);
-          a = *(src_address++);
-          *(dst_address++) = rgba(r, g, b, a);
-        }
-      }
-      /* RGB */
-      else if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB) {
-        uint8_t* src_address = row_pointer;
-        uint32_t* dst_address = (uint32_t*)image->getPixelAddress(0, y);
-        unsigned int x, r, g, b;
-
-        for (x=0; x<width; x++) {
-          r = *(src_address++);
-          g = *(src_address++);
-          b = *(src_address++);
-          *(dst_address++) = rgba(r, g, b, 255);
-        }
-      }
-      /* GRAY_ALPHA */
-      else if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY_ALPHA) {
-        uint8_t* src_address = row_pointer;
-        uint16_t* dst_address = (uint16_t*)image->getPixelAddress(0, y);
-        unsigned int x, k, a;
-
-        for (x=0; x<width; x++) {
-          k = *(src_address++);
-          a = *(src_address++);
-          *(dst_address++) = graya(k, a);
-        }
-      }
-      /* GRAY */
-      else if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY) {
-        uint8_t* src_address = row_pointer;
-        uint16_t* dst_address = (uint16_t*)image->getPixelAddress(0, y);
-        unsigned int x, k;
-
-        for (x=0; x<width; x++) {
-          k = *(src_address++);
-          *(dst_address++) = graya(k, 255);
-        }
-      }
-      /* PALETTE */
-      else if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE) {
-        uint8_t* src_address = row_pointer;
-        uint8_t* dst_address = (uint8_t*)image->getPixelAddress(0, y);
-        unsigned int x, c;
-
-        for (x=0; x<width; x++) {
-          c = *(src_address++);
-
-          if (pal_alphas[c] < 128) {
-            *(dst_address++) = mask_entry;
-          }
-          else {
-            *(dst_address++) = c;
-          }
-        }
-      }
-
-      fop_progress(fop,
-                   (double)((double)pass + (double)(y+1) / (double)(height))
-                   / (double)number_passes);
-
-      if (fop_is_stop(fop))
+      if (fop->isStop())
         break;
     }
   }
-  png_free(png_ptr, row_pointer);
 
-  /* clean up after the read, and free any memory allocated */
+  // Convert rows_pointer into the doc::Image
+  for (y = 0; y < height; y++) {
+    // RGB_ALPHA
+    if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB_ALPHA) {
+      uint8_t* src_address = rows_pointer[y];
+      uint32_t* dst_address = (uint32_t*)image->getPixelAddress(0, y);
+      unsigned int x, r, g, b, a;
+
+      for (x=0; x<width; x++) {
+        r = *(src_address++);
+        g = *(src_address++);
+        b = *(src_address++);
+        a = *(src_address++);
+        *(dst_address++) = rgba(r, g, b, a);
+      }
+    }
+    // RGB
+    else if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_RGB) {
+      uint8_t* src_address = rows_pointer[y];
+      uint32_t* dst_address = (uint32_t*)image->getPixelAddress(0, y);
+      unsigned int x, r, g, b, a;
+
+      for (x=0; x<width; x++) {
+        r = *(src_address++);
+        g = *(src_address++);
+        b = *(src_address++);
+
+        // Transparent color
+        if (png_trans_color &&
+            r == png_trans_color->red &&
+            g == png_trans_color->green &&
+            b == png_trans_color->blue) {
+          a = 0;
+          if (!fop->sequenceGetHasAlpha())
+            fop->sequenceSetHasAlpha(true);
+        }
+        else
+          a = 255;
+
+        *(dst_address++) = rgba(r, g, b, a);
+      }
+    }
+    // GRAY_ALPHA
+    else if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY_ALPHA) {
+      uint8_t* src_address = rows_pointer[y];
+      uint16_t* dst_address = (uint16_t*)image->getPixelAddress(0, y);
+      unsigned int x, k, a;
+
+      for (x=0; x<width; x++) {
+        k = *(src_address++);
+        a = *(src_address++);
+        *(dst_address++) = graya(k, a);
+      }
+    }
+    // GRAY
+    else if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_GRAY) {
+      uint8_t* src_address = rows_pointer[y];
+      uint16_t* dst_address = (uint16_t*)image->getPixelAddress(0, y);
+      unsigned int x, k, a;
+
+      for (x=0; x<width; x++) {
+        k = *(src_address++);
+
+        // Transparent color
+        if (png_trans_color &&
+            k == png_trans_color->gray) {
+          a = 0;
+          if (!fop->sequenceGetHasAlpha())
+            fop->sequenceSetHasAlpha(true);
+        }
+        else
+          a = 255;
+
+        *(dst_address++) = graya(k, a);
+      }
+    }
+    // PALETTE
+    else if (png_get_color_type(png_ptr, info_ptr) == PNG_COLOR_TYPE_PALETTE) {
+      uint8_t* src_address = rows_pointer[y];
+      uint8_t* dst_address = (uint8_t*)image->getPixelAddress(0, y);
+      unsigned int x;
+
+      for (x=0; x<width; x++)
+        *(dst_address++) = *(src_address++);
+    }
+    png_free(png_ptr, rows_pointer[y]);
+  }
+  png_free(png_ptr, rows_pointer);
+
+  // Clean up after the read, and free any memory allocated
   png_destroy_read_struct(&png_ptr, &info_ptr, NULL);
   return true;
 }
@@ -332,7 +341,7 @@ bool PngFormat::onLoad(FileOp* fop)
 #ifdef ENABLE_SAVE
 bool PngFormat::onSave(FileOp* fop)
 {
-  Image *image = fop->seq.image;
+  const Image* image = fop->sequenceImage();
   png_uint_32 width, height, y;
   png_structp png_ptr;
   png_infop info_ptr;
@@ -342,7 +351,8 @@ bool PngFormat::onSave(FileOp* fop)
   int pass, number_passes;
 
   /* open the file */
-  FileHandle fp(open_file_with_exception(fop->filename, "wb"));
+  FileHandle handle(open_file_with_exception(fop->filename(), "wb"));
+  FILE* fp = handle.get();
 
   /* Create and initialize the png_struct with the desired error handler
    * functions.  If you want to use the default stderr and longjump method,
@@ -388,12 +398,12 @@ bool PngFormat::onSave(FileOp* fop)
 
   switch (image->pixelFormat()) {
     case IMAGE_RGB:
-      color_type = fop->document->sprite()->needAlpha() ?
+      color_type = fop->document()->sprite()->needAlpha() ?
         PNG_COLOR_TYPE_RGB_ALPHA:
         PNG_COLOR_TYPE_RGB;
       break;
     case IMAGE_GRAYSCALE:
-      color_type = fop->document->sprite()->needAlpha() ?
+      color_type = fop->document()->sprite()->needAlpha() ?
         PNG_COLOR_TYPE_GRAY_ALPHA:
         PNG_COLOR_TYPE_GRAY;
       break;
@@ -407,36 +417,43 @@ bool PngFormat::onSave(FileOp* fop)
 
   if (image->pixelFormat() == IMAGE_INDEXED) {
     int c, r, g, b;
+    int pal_size = fop->sequenceGetNColors();
+    pal_size = MID(1, pal_size, PNG_MAX_PALETTE_LENGTH);
 
 #if PNG_MAX_PALETTE_LENGTH != 256
 #error PNG_MAX_PALETTE_LENGTH should be 256
 #endif
 
     // Save the color palette.
-    palette = (png_colorp)png_malloc(png_ptr, PNG_MAX_PALETTE_LENGTH * png_sizeof(png_color));
-    for (c = 0; c < PNG_MAX_PALETTE_LENGTH; c++) {
-      fop_sequence_get_color(fop, c, &r, &g, &b);
+    palette = (png_colorp)png_malloc(png_ptr, pal_size * sizeof(png_color));
+    for (c = 0; c < pal_size; c++) {
+      fop->sequenceGetColor(c, &r, &g, &b);
       palette[c].red   = r;
       palette[c].green = g;
       palette[c].blue  = b;
     }
 
-    png_set_PLTE(png_ptr, info_ptr, palette, PNG_MAX_PALETTE_LENGTH);
+    png_set_PLTE(png_ptr, info_ptr, palette, pal_size);
 
-    // If the sprite does not have a background layer, we include the
-    // alpha information of palette entries to indicate which is the
-    // transparent color.
-    if (fop->document->sprite()->backgroundLayer() == NULL) {
-      int mask_entry = fop->document->sprite()->transparentColor();
-      int num_trans = mask_entry+1;
-      png_bytep trans = (png_bytep)png_malloc(png_ptr, num_trans);
-
-      for (c = 0; c < num_trans; ++c)
-        trans[c] = (c == mask_entry ? 0: 255);
-
-      png_set_tRNS(png_ptr, info_ptr, trans, num_trans, NULL);
-      png_free(png_ptr, trans);
+    // If the sprite does not have a (visible) background layer, we
+    // put alpha=0 to the transparent color.
+    int mask_entry = -1;
+    if (fop->document()->sprite()->backgroundLayer() == NULL ||
+        !fop->document()->sprite()->backgroundLayer()->isVisible()) {
+      mask_entry = fop->document()->sprite()->transparentColor();
     }
+
+    int num_trans = pal_size;
+    png_bytep trans = (png_bytep)png_malloc(png_ptr, num_trans);
+
+    for (c=0; c<num_trans; ++c) {
+      int alpha = 255;
+      fop->sequenceGetAlpha(c, &alpha);
+      trans[c] = (c == mask_entry ? 0: alpha);
+    }
+
+    png_set_tRNS(png_ptr, info_ptr, trans, num_trans, NULL);
+    png_free(png_ptr, trans);
   }
 
   /* Write the file header information. */
@@ -519,9 +536,9 @@ bool PngFormat::onSave(FileOp* fop)
       /* write the line */
       png_write_rows(png_ptr, &row_pointer, 1);
 
-      fop_progress(fop,
-                   (double)((double)pass + (double)(y+1) / (double)(height))
-                   / (double)number_passes);
+      fop->setProgress(
+        (double)((double)pass + (double)(y+1) / (double)(height))
+        / (double)number_passes);
     }
   }
 
