@@ -1,24 +1,14 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2015  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 /* Some of the original code to handle PIDLs come from the
    MiniExplorer example of the Vaca library:
-   http://vaca.sourceforge.net/
+   https://github.com/dacap/vaca
+   Copyright (C) by David Capello (MIT License)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,7 +20,9 @@
 #include "base/fs.h"
 #include "base/path.h"
 #include "base/string.h"
+#include "she/display.h"
 #include "she/surface.h"
+#include "she/system.h"
 
 #include <algorithm>
 #include <cstdio>
@@ -38,61 +30,15 @@
 #include <utility>
 #include <vector>
 
-#include <allegro.h>
-
-// define this macro to solve the problem of for_each_file Allegro
-// routine which does not support to wrap 64-bits pointers in its
-// user-data parameter
-#define WORKAROUND_64BITS_SUPPORT
-
-// in Windows we can use PIDLS
-#if defined ALLEGRO_WINDOWS
-  // uncomment this if you don't want to use PIDLs in windows
-  #define USE_PIDLS
-#endif
-
-#if defined ALLEGRO_UNIX || defined ALLEGRO_MACOSX || defined ALLEGRO_DJGPP || defined ALLEGRO_MINGW32
-  #include <sys/stat.h>
-#endif
-#if defined ALLEGRO_UNIX || defined ALLEGRO_MACOSX || defined ALLEGRO_MINGW32
-  #include <sys/unistd.h>
-
-#endif
-
-#if defined USE_PIDLS
-  #include <winalleg.h>
+#ifdef _WIN32
+  #include <windows.h>
 
   #include <shlobj.h>
   #include <shlwapi.h>
-#endif
-
-//////////////////////////////////////////////////////////////////////
-
-#ifdef USE_PIDLS
-  // ..using Win32 Shell (PIDLs)
-
-  #define IS_FOLDER(fi)                                 \
-    (((fi)->attrib & SFGAO_FOLDER) == SFGAO_FOLDER)
 
   #define MYPC_CSLID  "::{20D04FE0-3AEA-1069-A2D8-08002B30309D}"
-
 #else
-  // ..using Allegro (for_each_file)
-
-  #define IS_FOLDER(fi)                                 \
-    (((fi)->attrib & FA_DIREC) == FA_DIREC)
-
-  #if (DEVICE_SEPARATOR != 0) && (DEVICE_SEPARATOR != '\0')
-    #define HAVE_DRIVES
-  #else
-    #define CASE_SENSITIVE
-  #endif
-
-  #ifndef FA_ALL
-    #define FA_ALL     FA_RDONLY | FA_DIREC | FA_ARCH | FA_HIDDEN | FA_SYSTEM
-  #endif
-  #define FA_TO_SHOW   FA_RDONLY | FA_DIREC | FA_ARCH | FA_SYSTEM
-
+  #include <dirent.h>
 #endif
 
 //////////////////////////////////////////////////////////////////////
@@ -108,21 +54,19 @@ namespace app {
 // a position in the file-system
 class FileItem : public IFileItem {
 public:
-  std::string keyname;
-  std::string filename;
-  std::string displayname;
-  FileItem* parent;
-  FileItemList children;
-  unsigned int version;
-  bool removed;
-#ifdef USE_PIDLS
-  LPITEMIDLIST pidl;            // relative to parent
-  LPITEMIDLIST fullpidl;        // relative to the Desktop folder
-                                // (like a full path-name, because the
-                                // desktop is the root on Windows)
-  SFGAOF attrib;
-#else
-  int attrib;
+  std::string m_keyname;
+  std::string m_filename;
+  std::string m_displayname;
+  FileItem* m_parent;
+  FileItemList m_children;
+  unsigned int m_version;
+  bool m_removed;
+  bool m_is_folder;
+#ifdef _WIN32
+  LPITEMIDLIST m_pidl;            // relative to parent
+  LPITEMIDLIST m_fullpidl;        // relative to the Desktop folder
+                                  // (like a full path-name, because the
+                                  // desktop is the root on Windows)
 #endif
 
   FileItem(FileItem* parent);
@@ -140,13 +84,14 @@ public:
 
   bool isFolder() const;
   bool isBrowsable() const;
+  bool isHidden() const;
 
-  std::string getKeyName() const;
-  std::string getFileName() const;
-  std::string getDisplayName() const;
+  std::string keyName() const;
+  std::string fileName() const;
+  std::string displayName() const;
 
-  IFileItem* getParent() const;
-  const FileItemList& getChildren();
+  IFileItem* parent() const;
+  const FileItemList& children();
   void createDirectory(const std::string& dirname);
 
   bool hasExtension(const std::string& csv_extensions);
@@ -165,18 +110,15 @@ static FileItemMap* fileitems_map;
 static ThumbnailMap* thumbnail_map;
 static unsigned int current_file_system_version = 0;
 
-#ifdef USE_PIDLS
+#ifdef _WIN32
   static IMalloc* shl_imalloc = NULL;
   static IShellFolder* shl_idesktop = NULL;
-#else
-  #ifdef WORKAROUND_64BITS_SUPPORT
-  static FileItem* for_each_child_callback_param;
-  #endif
 #endif
 
-/* a more easy PIDLs interface (without using the SH* & IL* routines of W2K) */
-#ifdef USE_PIDLS
-  static void update_by_pidl(FileItem* fileitem);
+// A more easy PIDLs interface (without using the SH* & IL* routines of W2K)
+#ifdef _WIN32
+  static bool is_sfgaof_folder(SFGAOF attrib);
+  static void update_by_pidl(FileItem* fileitem, SFGAOF attrib);
   static LPITEMIDLIST concat_pidl(LPITEMIDLIST pidlHead, LPITEMIDLIST pidlTail);
   static UINT get_pidl_size(LPITEMIDLIST pidl);
   static LPITEMIDLIST get_next_pidl(LPITEMIDLIST pidl);
@@ -190,7 +132,6 @@ static unsigned int current_file_system_version = 0;
   static void put_fileitem(FileItem* fileitem);
 #else
   static FileItem* get_fileitem_by_path(const std::string& path, bool create_if_not);
-  static void for_each_child_callback(const char *filename, int attrib, int param);
   static std::string remove_backslash_if_needed(const std::string& filename);
   static std::string get_key_for_filename(const std::string& filename);
   static void put_fileitem(FileItem* fileitem);
@@ -206,7 +147,7 @@ FileSystemModule::FileSystemModule()
   fileitems_map = new FileItemMap;
   thumbnail_map = new ThumbnailMap;
 
-#ifdef USE_PIDLS
+#ifdef _WIN32
   /* get the IMalloc interface */
   HRESULT hr = SHGetMalloc(&shl_imalloc);
   if (hr != S_OK)
@@ -225,12 +166,12 @@ FileSystemModule::FileSystemModule()
   // the 'rootitem' FileItem)
   getRootFileItem();
 
-  PRINTF("File system module installed\n");
+  LOG("File system module installed\n");
 }
 
 FileSystemModule::~FileSystemModule()
 {
-  PRINTF("File system module: uninstalling\n");
+  LOG("File system module: uninstalling\n");
   ASSERT(m_instance == this);
 
   for (FileItemMap::iterator
@@ -245,7 +186,7 @@ FileSystemModule::~FileSystemModule()
   }
   thumbnail_map->clear();
 
-#ifdef USE_PIDLS
+#ifdef _WIN32
   // relase desktop IShellFolder interface
   shl_idesktop->Release();
 
@@ -257,7 +198,7 @@ FileSystemModule::~FileSystemModule()
   delete fileitems_map;
   delete thumbnail_map;
 
-  PRINTF("File system module: uninstalled\n");
+  LOG("File system module: uninstalled\n");
   m_instance = NULL;
 }
 
@@ -281,9 +222,9 @@ IFileItem* FileSystemModule::getRootFileItem()
   fileitem = new FileItem(NULL);
   rootitem = fileitem;
 
-  //PRINTF("FS: Creating root fileitem %p\n", rootitem);
+  //LOG("FS: Creating root fileitem %p\n", rootitem);
 
-#ifdef USE_PIDLS
+#ifdef _WIN32
   {
     // get the desktop PIDL
     LPITEMIDLIST pidl = NULL;
@@ -293,26 +234,21 @@ IFileItem* FileSystemModule::getRootFileItem()
       ASSERT(false);
       exit(1);
     }
-    fileitem->pidl = pidl;
-    fileitem->fullpidl = pidl;
-    fileitem->attrib = SFGAO_FOLDER;
-    shl_idesktop->GetAttributesOf(1, (LPCITEMIDLIST *)&pidl, &fileitem->attrib);
+    fileitem->m_pidl = pidl;
+    fileitem->m_fullpidl = pidl;
 
-    update_by_pidl(fileitem);
+    SFGAOF attrib = SFGAO_FOLDER;
+    shl_idesktop->GetAttributesOf(1, (LPCITEMIDLIST *)&pidl, &attrib);
+
+    update_by_pidl(fileitem, attrib);
   }
 #else
   {
-    const char* root;
+    const char* root = "/";
 
-#if defined HAVE_DRIVES
-    root = "C:\\";
-#else
-    root = "/";
-#endif
-
-    fileitem->filename = root;
-    fileitem->displayname = root;
-    fileitem->attrib = FA_DIREC;
+    fileitem->m_filename = root;
+    fileitem->m_displayname = root;
+    fileitem->m_is_folder = true;
   }
 #endif
 
@@ -325,9 +261,9 @@ IFileItem* FileSystemModule::getFileItemFromPath(const std::string& path)
 {
   IFileItem* fileitem = NULL;
 
-  //PRINTF("FS: get_fileitem_from_path(%s)\n", path.c_str());
+  //LOG("FS: get_fileitem_from_path(%s)\n", path.c_str());
 
-#ifdef USE_PIDLS
+#ifdef _WIN32
   {
     ULONG cbEaten = 0UL;
     LPITEMIDLIST fullpidl = NULL;
@@ -335,7 +271,7 @@ IFileItem* FileSystemModule::getFileItemFromPath(const std::string& path)
 
     if (path.empty()) {
       fileitem = getRootFileItem();
-      //PRINTF("FS: > %p (root)\n", fileitem);
+      //LOG("FS: > %p (root)\n", fileitem);
       return fileitem;
     }
 
@@ -343,7 +279,7 @@ IFileItem* FileSystemModule::getFileItemFromPath(const std::string& path)
           (NULL, NULL,
            const_cast<LPWSTR>(base::from_utf8(path).c_str()),
            &cbEaten, &fullpidl, &attrib) != S_OK) {
-      //PRINTF("FS: > (null)\n");
+      //LOG("FS: > (null)\n");
       return NULL;
     }
 
@@ -357,98 +293,90 @@ IFileItem* FileSystemModule::getFileItemFromPath(const std::string& path)
   }
 #endif
 
-  //PRINTF("FS: get_fileitem_from_path(%s) -> %p\n", path.c_str(), fileitem);
+  //LOG("FS: get_fileitem_from_path(%s) -> %p\n", path.c_str(), fileitem);
 
   return fileitem;
 }
 
-bool FileSystemModule::dirExists(const std::string& path)
-{
-  struct al_ffblk info;
-  int ret;
-  std::string path2 = base::join_path(path, "*.*");
-
-  ret = al_findfirst(path2.c_str(), &info, FA_ALL);
-  al_findclose(&info);
-
-  return (ret == 0);
-}
-
 // ======================================================================
 // FileItem class (IFileItem implementation)
 // ======================================================================
 
 bool FileItem::isFolder() const
 {
-  return IS_FOLDER(this);
+  return m_is_folder;
 }
 
 bool FileItem::isBrowsable() const
 {
-  ASSERT(this->filename != NOTINITIALIZED);
+  ASSERT(m_filename != NOTINITIALIZED);
 
-#ifdef USE_PIDLS
-  return IS_FOLDER(this)
-    && (base::get_file_extension(this->filename) != "zip")
-    && ((!this->filename.empty() && (*this->filename.begin()) != ':') ||
-        (this->filename == MYPC_CSLID));
+  return m_is_folder;
+}
+
+bool FileItem::isHidden() const
+{
+  ASSERT(m_displayname != NOTINITIALIZED);
+
+#ifdef _WIN32
+  return false;
 #else
-  return IS_FOLDER(this);
+  return m_displayname[0] == '.';
 #endif
 }
 
-std::string FileItem::getKeyName() const
+std::string FileItem::keyName() const
 {
-  ASSERT(this->keyname != NOTINITIALIZED);
+  ASSERT(m_keyname != NOTINITIALIZED);
 
-  return this->keyname;
+  return m_keyname;
 }
 
-std::string FileItem::getFileName() const
+std::string FileItem::fileName() const
 {
-  ASSERT(this->filename != NOTINITIALIZED);
+  ASSERT(m_filename != NOTINITIALIZED);
 
-  return this->filename;
+  return m_filename;
 }
 
-std::string FileItem::getDisplayName() const
+std::string FileItem::displayName() const
 {
-  ASSERT(this->displayname != NOTINITIALIZED);
+  ASSERT(m_displayname != NOTINITIALIZED);
 
-  return this->displayname;
+  return m_displayname;
 }
 
-IFileItem* FileItem::getParent() const
+IFileItem* FileItem::parent() const
 {
   if (this == rootitem)
     return NULL;
   else {
-    ASSERT(this->parent);
-    return this->parent;
+    ASSERT(m_parent);
+    return m_parent;
   }
 }
 
-const FileItemList& FileItem::getChildren()
+const FileItemList& FileItem::children()
 {
   // Is the file-item a folder?
-  if (IS_FOLDER(this) &&
+  if (isFolder() &&
       // if the children list is empty, or the file-system version
-      // change (it's like to say: the current this->children list
+      // change (it's like to say: the current m_children list
       // is outdated)...
-      (this->children.empty() ||
-       current_file_system_version > this->version)) {
+      (m_children.empty() ||
+       current_file_system_version > m_version)) {
     FileItemList::iterator it;
     FileItem* child;
 
     // we have to mark current items as deprecated
-    for (it=this->children.begin();
-         it!=this->children.end(); ++it) {
+    for (it=m_children.begin();
+         it!=m_children.end(); ++it) {
       child = static_cast<FileItem*>(*it);
-      child->removed = true;
+      child->m_removed = true;
     }
 
-    //PRINTF("FS: Loading files for %p (%s)\n", fileitem, fileitem->displayname);
-#ifdef USE_PIDLS
+    //LOG("FS: Loading files for %p (%s)\n", fileitem, fileitem->displayname);
+#ifdef _WIN32
     {
       IShellFolder* pFolder = NULL;
       HRESULT hr;
@@ -456,7 +384,7 @@ const FileItemList& FileItem::getChildren()
       if (this == rootitem)
         pFolder = shl_idesktop;
       else {
-        hr = shl_idesktop->BindToObject(this->fullpidl,
+        hr = shl_idesktop->BindToObject(m_fullpidl,
           NULL, IID_IShellFolder, (LPVOID *)&pFolder);
 
         if (hr != S_OK)
@@ -468,7 +396,7 @@ const FileItemList& FileItem::getChildren()
         ULONG c, fetched;
 
         /* get the interface to enumerate subitems */
-        hr = pFolder->EnumObjects(win_get_window(),
+        hr = pFolder->EnumObjects(reinterpret_cast<HWND>(she::instance()->defaultDisplay()->nativeHandle()),
           SHCONTF_FOLDERS | SHCONTF_NONFOLDERS, &pEnum);
 
         if (hr == S_OK && pEnum != NULL) {
@@ -486,27 +414,26 @@ const FileItemList& FileItem::getChildren()
 
             /* generate the FileItems */
             for (c=0; c<fetched; ++c) {
-              LPITEMIDLIST fullpidl = concat_pidl(this->fullpidl,
+              LPITEMIDLIST fullpidl = concat_pidl(m_fullpidl,
                                                   itempidl[c]);
 
               child = get_fileitem_by_fullpidl(fullpidl, false);
               if (!child) {
                 child = new FileItem(this);
 
-                child->pidl = itempidl[c];
-                child->fullpidl = fullpidl;
-                child->attrib = attribs[c];
+                child->m_pidl = itempidl[c];
+                child->m_fullpidl = fullpidl;
 
-                update_by_pidl(child);
+                update_by_pidl(child, attribs[c]);
                 put_fileitem(child);
               }
               else {
-                ASSERT(child->parent == this);
+                ASSERT(child->m_parent == this);
                 free_pidl(fullpidl);
                 free_pidl(itempidl[c]);
               }
 
-              this->insertChildSorted(child);
+              insertChildSorted(child);
             }
           }
 
@@ -519,38 +446,56 @@ const FileItemList& FileItem::getChildren()
     }
 #else
     {
-      char buf[MAX_PATH], path[MAX_PATH], tmp[32];
+      DIR* dir = opendir(m_filename.c_str());
+      if (dir) {
+        dirent* entry;
+        while ((entry = readdir(dir)) != NULL) {
+          FileItem* child;
+          std::string fn = entry->d_name;
+          std::string fullfn = base::join_path(m_filename, fn);
 
-      ustrcpy(path, this->filename.c_str());
-      put_backslash(path);
+          if (fn == "." || fn == "..")
+            continue;
 
-      replace_filename(buf,
-                       path,
-                       uconvert_ascii("*.*", tmp),
-                       sizeof(buf));
+          child = get_fileitem_by_path(fullfn, false);
+          if (!child) {
+            child = new FileItem(this);
 
-#ifdef WORKAROUND_64BITS_SUPPORT
-      // we cannot use the for_each_file's 'param' to wrap a 64-bits pointer
-      for_each_child_callback_param = this;
-      for_each_file(buf, FA_TO_SHOW, for_each_child_callback, 0);
-#else
-      for_each_file(buf, FA_TO_SHOW,
-                    for_each_child_callback,
-                    (int)this);
-#endif
+            bool is_folder;
+            if (entry->d_type == DT_LNK) {
+              is_folder = base::is_directory(fullfn);
+            }
+            else {
+              is_folder = (entry->d_type == DT_DIR);
+            }
+
+            child->m_filename = fullfn;
+            child->m_displayname = fn;
+            child->m_is_folder = is_folder;
+
+            put_fileitem(child);
+          }
+          else {
+            ASSERT(child->m_parent == this);
+          }
+
+          insertChildSorted(child);
+        }
+        closedir(dir);
+      }
   }
 #endif
 
     // check old file-items (maybe removed directories or file-items)
-    for (it=this->children.begin();
-         it!=this->children.end(); ) {
+    for (it=m_children.begin();
+         it!=m_children.end(); ) {
       child = static_cast<FileItem*>(*it);
       ASSERT(child != NULL);
 
-      if (child && child->removed) {
-        it = this->children.erase(it);
+      if (child && child->m_removed) {
+        it = m_children.erase(it);
 
-        fileitems_map->erase(fileitems_map->find(child->keyname));
+        fileitems_map->erase(fileitems_map->find(child->m_keyname));
         delete child;
       }
       else
@@ -558,30 +503,30 @@ const FileItemList& FileItem::getChildren()
     }
 
     // now this file-item is updated
-    this->version = current_file_system_version;
+    m_version = current_file_system_version;
   }
 
-  return this->children;
+  return m_children;
 }
 
 void FileItem::createDirectory(const std::string& dirname)
 {
-  base::make_directory(base::join_path(filename, dirname));
+  base::make_directory(base::join_path(m_filename, dirname));
 
   // Invalidate the children list.
-  this->version = 0;
+  m_version = 0;
 }
 
 bool FileItem::hasExtension(const std::string& csv_extensions)
 {
-  ASSERT(this->filename != NOTINITIALIZED);
+  ASSERT(m_filename != NOTINITIALIZED);
 
-  return base::has_file_extension(this->filename, csv_extensions);
+  return base::has_file_extension(m_filename, csv_extensions);
 }
 
 she::Surface* FileItem::getThumbnail()
 {
-  ThumbnailMap::iterator it = thumbnail_map->find(this->filename);
+  ThumbnailMap::iterator it = thumbnail_map->find(m_filename);
   if (it != thumbnail_map->end())
     return it->second;
   else
@@ -591,48 +536,46 @@ she::Surface* FileItem::getThumbnail()
 void FileItem::setThumbnail(she::Surface* thumbnail)
 {
   // destroy the current thumbnail of the file (if exists)
-  ThumbnailMap::iterator it = thumbnail_map->find(this->filename);
+  ThumbnailMap::iterator it = thumbnail_map->find(m_filename);
   if (it != thumbnail_map->end()) {
     it->second->dispose();
     thumbnail_map->erase(it);
   }
 
   // insert the new one in the map
-  thumbnail_map->insert(std::make_pair(this->filename, thumbnail));
+  thumbnail_map->insert(std::make_pair(m_filename, thumbnail));
 }
 
 FileItem::FileItem(FileItem* parent)
 {
-  //PRINTF("FS: Creating %p fileitem with parent %p\n", this, parent);
+  //LOG("FS: Creating %p fileitem with parent %p\n", this, parent);
 
-  this->keyname = NOTINITIALIZED;
-  this->filename = NOTINITIALIZED;
-  this->displayname = NOTINITIALIZED;
-  this->parent = parent;
-  this->version = current_file_system_version;
-  this->removed = false;
-#ifdef USE_PIDLS
-  this->pidl = NULL;
-  this->fullpidl = NULL;
-  this->attrib = 0;
-#else
-  this->attrib = 0;
+  m_keyname = NOTINITIALIZED;
+  m_filename = NOTINITIALIZED;
+  m_displayname = NOTINITIALIZED;
+  m_parent = parent;
+  m_version = current_file_system_version;
+  m_removed = false;
+  m_is_folder = false;
+#ifdef _WIN32
+  m_pidl = NULL;
+  m_fullpidl = NULL;
 #endif
 }
 
 FileItem::~FileItem()
 {
-  PRINTF("FS: Destroying FileItem() with parent %p\n", parent);
+  LOG("FS: Destroying FileItem() with parent %p\n", m_parent);
 
-#ifdef USE_PIDLS
-  if (this->fullpidl && this->fullpidl != this->pidl) {
-    free_pidl(this->fullpidl);
-    this->fullpidl = NULL;
+#ifdef _WIN32
+  if (m_fullpidl && m_fullpidl != m_pidl) {
+    free_pidl(m_fullpidl);
+    m_fullpidl = NULL;
   }
 
-  if (this->pidl) {
-    free_pidl(this->pidl);
-    this->pidl = NULL;
+  if (m_pidl) {
+    free_pidl(m_pidl);
+    m_pidl = NULL;
   }
 #endif
 }
@@ -640,147 +583,106 @@ FileItem::~FileItem()
 void FileItem::insertChildSorted(FileItem* child)
 {
   // this file-item wasn't removed from the last lookup
-  child->removed = false;
+  child->m_removed = false;
 
   // if the fileitem is already in the list we can go back
-  if (std::find(children.begin(), children.end(), child) != children.end())
+  if (std::find(m_children.begin(), m_children.end(), child) != m_children.end())
     return;
 
-  bool inserted = false;
-
-  for (FileItemList::iterator
-         it=children.begin(); it!=children.end(); ++it) {
-    if (*static_cast<FileItem*>(*it) > *child) {
-      children.insert(it, child);
-      inserted = true;
-      break;
+  for (auto it=m_children.begin(), end=m_children.end(); it!=end; ++it) {
+    if (*child < *static_cast<FileItem*>(*it)) {
+      m_children.insert(it, child);
+      return;
     }
   }
 
-  if (!inserted)
-    children.push_back(child);
+  m_children.push_back(child);
 }
 
-/**
- * Compares two FileItems.
- *
- * Based on 'ustricmp' of Allegro. It makes sure that eg "foo.bar"
- * comes before "foo-1.bar", and also that "foo9.bar" comes before
- * "foo10.bar".
- */
 int FileItem::compare(const FileItem& that) const
 {
-  if (IS_FOLDER(this)) {
-    if (!IS_FOLDER(&that))
+  if (isFolder()) {
+    if (!that.isFolder())
       return -1;
   }
-  else if (IS_FOLDER(&that))
+  else if (that.isFolder())
     return 1;
 
-  {
-    int c1, c2;
-    int x1, x2;
-    char *t1, *t2;
-    const char* s1 = this->displayname.c_str();
-    const char* s2 = that.displayname.c_str();
-
-    for (;;) {
-      c1 = utolower(ugetxc(&s1));
-      c2 = utolower(ugetxc(&s2));
-
-      if ((c1 >= '0') && (c1 <= '9') && (c2 >= '0') && (c2 <= '9')) {
-        x1 = ustrtol(s1 - ucwidth(c1), &t1, 10);
-        x2 = ustrtol(s2 - ucwidth(c2), &t2, 10);
-        if (x1 != x2)
-          return x1 - x2;
-        else if (t1 - s1 != t2 - s2)
-          return (t2 - s2) - (t1 - s1);
-        s1 = t1;
-        s2 = t2;
-      }
-      else if (c1 != c2) {
-        if (!c1)
-          return -1;
-        else if (!c2)
-          return 1;
-        else if (c1 == '.')
-          return -1;
-        else if (c2 == '.')
-          return 1;
-        return c1 - c2;
-      }
-
-      if (!c1)
-        return 0;
-    }
-  }
-
-  return -1;
+  return base::compare_filenames(m_displayname, that.m_displayname);
 }
 
 //////////////////////////////////////////////////////////////////////
 // PIDLS: Only for Win32
 //////////////////////////////////////////////////////////////////////
 
-#ifdef USE_PIDLS
+#ifdef _WIN32
 
-/* updates the names of the file-item through its PIDL */
-static void update_by_pidl(FileItem* fileitem)
+static bool calc_is_folder(std::string filename, SFGAOF attrib)
+{
+  return ((attrib & SFGAO_FOLDER) == SFGAO_FOLDER)
+    && (base::get_file_extension(filename) != "zip")
+    && ((!filename.empty() && (*filename.begin()) != ':') || (filename == MYPC_CSLID));
+}
+
+// Updates the names of the file-item through its PIDL
+static void update_by_pidl(FileItem* fileitem, SFGAOF attrib)
 {
   STRRET strret;
   WCHAR pszName[MAX_PATH];
-  IShellFolder *pFolder = NULL;
+  IShellFolder* pFolder = NULL;
   HRESULT hr;
 
   if (fileitem == rootitem)
     pFolder = shl_idesktop;
   else {
-    ASSERT(fileitem->parent);
-    hr = shl_idesktop->BindToObject(fileitem->parent->fullpidl,
+    ASSERT(fileitem->m_parent);
+    hr = shl_idesktop->BindToObject(fileitem->m_parent->m_fullpidl,
       NULL, IID_IShellFolder, (LPVOID *)&pFolder);
     if (hr != S_OK)
       pFolder = NULL;
   }
 
-  /****************************************/
-  /* get the file name */
+  // Get the file name
 
   if (pFolder != NULL &&
-      pFolder->GetDisplayNameOf(fileitem->pidl,
+      pFolder->GetDisplayNameOf(fileitem->m_pidl,
                                 SHGDN_NORMAL | SHGDN_FORPARSING,
                                 &strret) == S_OK) {
-    StrRetToBuf(&strret, fileitem->pidl, pszName, MAX_PATH);
-    fileitem->filename = base::to_utf8(pszName);
+    StrRetToBuf(&strret, fileitem->m_pidl, pszName, MAX_PATH);
+    fileitem->m_filename = base::to_utf8(pszName);
   }
-  else if (shl_idesktop->GetDisplayNameOf(fileitem->fullpidl,
+  else if (shl_idesktop->GetDisplayNameOf(fileitem->m_fullpidl,
                                           SHGDN_NORMAL | SHGDN_FORPARSING,
                                           &strret) == S_OK) {
-    StrRetToBuf(&strret, fileitem->fullpidl, pszName, MAX_PATH);
-    fileitem->filename = base::to_utf8(pszName);
+    StrRetToBuf(&strret, fileitem->m_fullpidl, pszName, MAX_PATH);
+    fileitem->m_filename = base::to_utf8(pszName);
   }
   else
-    fileitem->filename = "ERR";
+    fileitem->m_filename = "ERR";
 
-  /****************************************/
-  /* get the name to display */
+  // Is it a folder?
+
+  fileitem->m_is_folder = calc_is_folder(fileitem->m_filename, attrib);
+
+  // Get the name to display
 
   if (fileitem->isFolder() &&
       pFolder &&
-      pFolder->GetDisplayNameOf(fileitem->pidl,
+      pFolder->GetDisplayNameOf(fileitem->m_pidl,
                                 SHGDN_INFOLDER,
                                 &strret) == S_OK) {
-    StrRetToBuf(&strret, fileitem->pidl, pszName, MAX_PATH);
-    fileitem->displayname = base::to_utf8(pszName);
+    StrRetToBuf(&strret, fileitem->m_pidl, pszName, MAX_PATH);
+    fileitem->m_displayname = base::to_utf8(pszName);
   }
   else if (fileitem->isFolder() &&
-           shl_idesktop->GetDisplayNameOf(fileitem->fullpidl,
+           shl_idesktop->GetDisplayNameOf(fileitem->m_fullpidl,
                                           SHGDN_INFOLDER,
                                           &strret) == S_OK) {
-    StrRetToBuf(&strret, fileitem->fullpidl, pszName, MAX_PATH);
-    fileitem->displayname = base::to_utf8(pszName);
+    StrRetToBuf(&strret, fileitem->m_fullpidl, pszName, MAX_PATH);
+    fileitem->m_displayname = base::to_utf8(pszName);
   }
   else {
-    fileitem->displayname = base::get_file_name(fileitem->filename);
+    fileitem->m_displayname = base::get_file_name(fileitem->m_filename);
   }
 
   if (pFolder != NULL && pFolder != shl_idesktop) {
@@ -910,7 +812,7 @@ static std::string get_key_for_pidl(LPITEMIDLIST pidl)
   int len;
 
   // Go pidl by pidl from the fullpidl to the root (desktop)
-  //PRINTF("FS: ***\n");
+  //LOG("FS: ***\n");
   pidl = clone_pidl(pidl);
   while (pidl->mkid.cb > 0) {
     if (shl_idesktop->GetDisplayNameOf(pidl,
@@ -919,7 +821,7 @@ static std::string get_key_for_pidl(LPITEMIDLIST pidl)
       if (StrRetToBuf(&strret, pidl, pszName, MAX_PATH) != S_OK)
         pszName[0] = 0;
 
-      //PRINTF("FS: + %s\n", pszName);
+      //LOG("FS: + %s\n", pszName);
 
       len = wcslen(pszName);
       if (len > 0) {
@@ -941,7 +843,7 @@ static std::string get_key_for_pidl(LPITEMIDLIST pidl)
   }
   free_pidl(pidl);
 
-  //PRINTF("FS: =%s\n***\n", key);
+  //LOG("FS: =%s\n***\n", key);
   return base::to_utf8(key);
 #endif
 }
@@ -957,25 +859,24 @@ static FileItem* get_fileitem_by_fullpidl(LPITEMIDLIST fullpidl, bool create_if_
 
   // new file-item
   FileItem* fileitem = new FileItem(NULL);
-  fileitem->fullpidl = clone_pidl(fullpidl);
+  fileitem->m_fullpidl = clone_pidl(fullpidl);
 
-  fileitem->attrib = SFGAO_FOLDER;
-  HRESULT hr = shl_idesktop->GetAttributesOf(1, (LPCITEMIDLIST *)&fileitem->fullpidl,
-                                             &fileitem->attrib);
+  SFGAOF attrib = SFGAO_FOLDER;
+  HRESULT hr = shl_idesktop->GetAttributesOf(1, (LPCITEMIDLIST *)&fileitem->m_fullpidl, &attrib);
   if (hr == S_OK) {
-    LPITEMIDLIST parent_fullpidl = clone_pidl(fileitem->fullpidl);
+    LPITEMIDLIST parent_fullpidl = clone_pidl(fileitem->m_fullpidl);
     remove_last_pidl(parent_fullpidl);
 
-    fileitem->pidl = get_last_pidl(fileitem->fullpidl);
-    fileitem->parent = get_fileitem_by_fullpidl(parent_fullpidl, true);
+    fileitem->m_pidl = get_last_pidl(fileitem->m_fullpidl);
+    fileitem->m_parent = get_fileitem_by_fullpidl(parent_fullpidl, true);
 
     free_pidl(parent_fullpidl);
   }
 
-  update_by_pidl(fileitem);
+  update_by_pidl(fileitem, attrib);
   put_fileitem(fileitem);
 
-  //PRINTF("FS: fileitem %p created %s with parent %p\n", fileitem, fileitem->keyname.c_str(), fileitem->parent);
+  //LOG("FS: fileitem %p created %s with parent %p\n", fileitem, fileitem->keyname.c_str(), fileitem->parent);
 
   return fileitem;
 }
@@ -985,26 +886,26 @@ static FileItem* get_fileitem_by_fullpidl(LPITEMIDLIST fullpidl, bool create_if_
  */
 static void put_fileitem(FileItem* fileitem)
 {
-  ASSERT(fileitem->filename != NOTINITIALIZED);
-  ASSERT(fileitem->keyname == NOTINITIALIZED);
+  ASSERT(fileitem->m_filename != NOTINITIALIZED);
+  ASSERT(fileitem->m_keyname == NOTINITIALIZED);
 
-  fileitem->keyname = get_key_for_pidl(fileitem->fullpidl);
+  fileitem->m_keyname = get_key_for_pidl(fileitem->m_fullpidl);
 
-  ASSERT(fileitem->keyname != NOTINITIALIZED);
+  ASSERT(fileitem->m_keyname != NOTINITIALIZED);
 
-#ifdef DEBUGMODE
-  FileItemMap::iterator it = fileitems_map->find(get_key_for_pidl(fileitem->fullpidl));
+#ifdef _DEBUG
+  auto it = fileitems_map->find(get_key_for_pidl(fileitem->m_fullpidl));
   ASSERT(it == fileitems_map->end());
 #endif
 
   // insert this file-item in the hash-table
-  fileitems_map->insert(std::make_pair(fileitem->keyname, fileitem));
+  fileitems_map->insert(std::make_pair(fileitem->m_keyname, fileitem));
 }
 
 #else
 
 //////////////////////////////////////////////////////////////////////
-// Allegro for_each_file: Portable
+// POSIX functions
 //////////////////////////////////////////////////////////////////////
 
 static FileItem* get_fileitem_by_path(const std::string& path, bool create_if_not)
@@ -1020,24 +921,25 @@ static FileItem* get_fileitem_by_path(const std::string& path, bool create_if_no
     return NULL;
 
   // get the attributes of the file
-  int attrib = 0;
-  if (!file_exists(path.c_str(), FA_ALL, &attrib)) {
-    if (!FileSystemModule::instance()->dirExists(path))
+  bool is_folder = false;
+  if (!base::is_file(path)) {
+    if (!base::is_directory(path))
       return NULL;
-    attrib = FA_DIREC;
+
+    is_folder = true;
   }
 
   // new file-item
   FileItem* fileitem = new FileItem(NULL);
 
-  fileitem->filename = path;
-  fileitem->displayname = base::get_file_name(path);
-  fileitem->attrib = attrib;
+  fileitem->m_filename = path;
+  fileitem->m_displayname = base::get_file_name(path);
+  fileitem->m_is_folder = is_folder;
 
   // get the parent
   {
     std::string parent_path = remove_backslash_if_needed(base::join_path(base::get_file_path(path), ""));
-    fileitem->parent = get_fileitem_by_path(parent_path, true);
+    fileitem->m_parent = get_fileitem_by_path(parent_path, true);
   }
 
   put_fileitem(fileitem);
@@ -1045,54 +947,16 @@ static FileItem* get_fileitem_by_path(const std::string& path, bool create_if_no
   return fileitem;
 }
 
-static void for_each_child_callback(const char *filename, int attrib, int param)
-{
-#ifdef WORKAROUND_64BITS_SUPPORT
-  FileItem* fileitem = for_each_child_callback_param;
-#else
-  FileItem* fileitem = (FileItem*)param;
-#endif
-  FileItem* child;
-  const char *filename_without_path = get_filename(filename);
-
-  if (*filename_without_path == '.' &&
-      (ustrcmp(filename_without_path, ".") == 0 ||
-       ustrcmp(filename_without_path, "..") == 0))
-    return;
-
-  child = get_fileitem_by_path(filename, false);
-  if (!child) {
-    ASSERT(fileitem != NULL);
-    child = new FileItem(fileitem);
-
-    child->filename = filename;
-    child->displayname = filename_without_path;
-    child->attrib = attrib;
-
-    put_fileitem(child);
-  }
-  else {
-    ASSERT(child->parent == fileitem);
-  }
-
-  fileitem->insertChildSorted(child);
-}
-
 static std::string remove_backslash_if_needed(const std::string& filename)
 {
   if (!filename.empty() && base::is_path_separator(*(filename.end()-1))) {
     int len = filename.size();
-#ifdef HAVE_DRIVES
-    // if the name is C:\ or something like that, the backslash isn't
-    // removed
-    if (len == 3 && filename[1] == ':')
-      return filename;
-#else
-    // this is just the root '/' slash
+
+    // This is just the root '/' slash
     if (len == 1)
       return filename;
-#endif
-    return base::remove_path_separator(filename);
+    else
+      return base::remove_path_separator(filename);
   }
   return filename;
 }
@@ -1100,26 +964,21 @@ static std::string remove_backslash_if_needed(const std::string& filename)
 static std::string get_key_for_filename(const std::string& filename)
 {
   std::string buf(filename);
-
-#if !defined CASE_SENSITIVE
-  buf = base::string_to_lower(buf);
-#endif
   buf = base::fix_path_separators(buf);
-
   return buf;
 }
 
 static void put_fileitem(FileItem* fileitem)
 {
-  ASSERT(fileitem->filename != NOTINITIALIZED);
-  ASSERT(fileitem->keyname == NOTINITIALIZED);
+  ASSERT(fileitem->m_filename != NOTINITIALIZED);
+  ASSERT(fileitem->m_keyname == NOTINITIALIZED);
 
-  fileitem->keyname = get_key_for_filename(fileitem->filename);
+  fileitem->m_keyname = get_key_for_filename(fileitem->m_filename);
 
-  ASSERT(fileitem->keyname != NOTINITIALIZED);
+  ASSERT(fileitem->m_keyname != NOTINITIALIZED);
 
   // insert this file-item in the hash-table
-  fileitems_map->insert(std::make_pair(fileitem->keyname, fileitem));
+  fileitems_map->insert(std::make_pair(fileitem->m_keyname, fileitem));
 }
 
 #endif
