@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2015  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -40,7 +29,7 @@ GuiXml* GuiXml::instance()
 
 GuiXml::GuiXml()
 {
-  PRINTF("Loading gui.xml file...");
+  LOG("Loading gui.xml file...\n");
 
   ResourceFinder rf;
   rf.includeDataDir("gui.xml");
@@ -54,7 +43,7 @@ GuiXml::GuiXml()
 
 std::string GuiXml::version()
 {
-  TiXmlHandle handle(m_doc);
+  TiXmlHandle handle(m_doc.get());
   TiXmlElement* xmlKey = handle.FirstChild("gui").ToElement();
 
   if (xmlKey && xmlKey->Attribute("version")) {
