@@ -1,31 +1,21 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2015  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifndef APP_RECENT_FILES_H_INCLUDED
 #define APP_RECENT_FILES_H_INCLUDED
 #pragma once
 
 #include "base/recent_items.h"
+#include "base/signal.h"
 
 #include <string>
 
 namespace app {
-  
+
   class RecentFiles {
   public:
     typedef base::RecentItems<std::string> List;
@@ -46,7 +36,11 @@ namespace app {
     void addRecentFile(const char* filename);
     void removeRecentFile(const char* filename);
 
+    base::Signal0<void> Changed;
+
   private:
+    std::string normalizePath(std::string fn);
+
     List m_files;
     List m_paths;
   };
