@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2014  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2015  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -30,7 +19,7 @@
 #include "base/fs.h"
 #include "base/path.h"
 #include "base/scoped_value.h"
-#include "raster/palette.h"
+#include "doc/palette.h"
 
 namespace app {
 
@@ -50,7 +39,7 @@ std::string PalettesLoaderDelegate::resourcesLocation() const
 
 Resource* PalettesLoaderDelegate::loadResource(const std::string& filename)
 {
-  raster::Palette* palette = load_palette(filename.c_str());
+  doc::Palette* palette = load_palette(filename.c_str());
   if (!palette)
     return NULL;
 
