@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifndef APP_TOOLS_TOOL_POINT_SHAPE_H_INCLUDED
 #define APP_TOOLS_TOOL_POINT_SHAPE_H_INCLUDED
@@ -30,8 +19,12 @@ namespace app {
     class PointShape {
     public:
       virtual ~PointShape() { }
+      virtual bool isPixel() { return false; }
       virtual bool isFloodFill() { return false; }
       virtual bool isSpray() { return false; }
+      virtual void preparePointShape(ToolLoop* loop) { }
+
+      // The x, y position must be relative to the cel/src/dst image origin.
       virtual void transformPoint(ToolLoop* loop, int x, int y) = 0;
       virtual void getModifiedArea(ToolLoop* loop, int x, int y, gfx::Rect& area) = 0;
 
