@@ -1,45 +1,38 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifndef APP_UI_EDITOR_STANDBY_STATE_H_INCLUDED
 #define APP_UI_EDITOR_STANDBY_STATE_H_INCLUDED
 #pragma once
 
 #include "app/ui/editor/editor_decorator.h"
-#include "app/ui/editor/editor_state.h"
 #include "app/ui/editor/handle_type.h"
+#include "app/ui/editor/state_with_wheel_behavior.h"
+#include "base/connection.h"
 #include "gfx/transformation.h"
 
 namespace app {
+  namespace tools {
+    class Ink;
+  }
+
   class TransformHandles;
 
-  class StandbyState : public EditorState {
+  class StandbyState : public StateWithWheelBehavior {
   public:
     StandbyState();
     virtual ~StandbyState();
-    virtual void onAfterChangeState(Editor* editor) override;
-    virtual void onCurrentToolChange(Editor* editor) override;
-    virtual void onQuickToolChange(Editor* editor) override;
+    virtual void onEnterState(Editor* editor) override;
+    virtual void onActiveToolChange(Editor* editor, tools::Tool* tool) override;
     virtual bool onMouseDown(Editor* editor, ui::MouseMessage* msg) override;
     virtual bool onMouseUp(Editor* editor, ui::MouseMessage* msg) override;
     virtual bool onMouseMove(Editor* editor, ui::MouseMessage* msg) override;
-    virtual bool onMouseWheel(Editor* editor, ui::MouseMessage* msg) override;
-    virtual bool onSetCursor(Editor* editor) override;
+    virtual bool onDoubleClick(Editor* editor, ui::MouseMessage* msg) override;
+    virtual bool onSetCursor(Editor* editor, const gfx::Point& mouseScreenPos) override;
     virtual bool onKeyDown(Editor* editor, ui::KeyMessage* msg) override;
     virtual bool onKeyUp(Editor* editor, ui::KeyMessage* msg) override;
     virtual bool onUpdateStatusBar(Editor* editor) override;
@@ -50,7 +43,7 @@ namespace app {
 
     virtual gfx::Transformation getTransformation(Editor* editor);
 
-    void startSelectionTransformation(Editor* editor, const gfx::Point& move);
+    void startSelectionTransformation(Editor* editor, const gfx::Point& move, double angle);
 
   protected:
     // Returns true and changes to ScrollingState when "msg" says "the
@@ -65,12 +58,15 @@ namespace app {
       virtual ~Decorator();
 
       TransformHandles* getTransformHandles(Editor* editor);
+      bool getSymmetryHandles(Editor* editor, gfx::Rect& box1, gfx::Rect& box2);
 
-      bool onSetCursor(Editor* editor);
+      bool onSetCursor(tools::Ink* ink, Editor* editor, const gfx::Point& mouseScreenPos);
 
       // EditorDecorator overrides
       void preRenderDecorator(EditorPreRender* render) override;
       void postRenderDecorator(EditorPostRender* render) override;
+      void getInvalidDecoratoredRegion(Editor* editor, gfx::Region& region) override;
+
     private:
       TransformHandles* m_transfHandles;
       StandbyState* m_standbyState;
@@ -78,8 +74,12 @@ namespace app {
 
   private:
     void transformSelection(Editor* editor, ui::MouseMessage* msg, HandleType handle);
+    void onPivotChange(Editor* editor);
 
     Decorator* m_decorator;
+    base::ScopedConnection m_pivotVisConn;
+    base::ScopedConnection m_pivotPosConn;
+    bool m_transformSelectionHandlesAreVisible;
   };
 
 } // namespace app
