@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2014  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifndef APP_UI_SELECT_ACCELERATOR_H_INCLUDED
 #define APP_UI_SELECT_ACCELERATOR_H_INCLUDED
@@ -22,8 +11,9 @@
 
 #include "app/ui/keyboard_shortcuts.h"
 #include "ui/accelerator.h"
+#include "ui/tooltips.h"
 
-#include "generated_select_accelerator.h"
+#include "select_accelerator.xml.h"
 
 namespace app {
 
@@ -45,6 +35,7 @@ namespace app {
 
     class KeyField;
 
+    ui::TooltipManager m_tooltipManager;
     KeyField* m_keyField;
     KeyContext m_keyContext;
     ui::Accelerator m_origAccel;
