@@ -1,5 +1,5 @@
 // Aseprite Base Library
-// Copyright (c) 2001-2013 David Capello
+// Copyright (c) 2001-2015 David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -8,12 +8,12 @@
 #define BASE_CONVERT_TO_H_INCLUDED
 #pragma once
 
+#include "base/base.h"
 #include <string>
 
 namespace base {
 
   class Sha1;
-  class Version;
 
   // Undefined convertion
   template<typename To, typename From>
@@ -24,12 +24,15 @@ namespace base {
   template<> int convert_to(const std::string& from);
   template<> std::string convert_to(const int& from);
 
+  template<> uint32_t convert_to(const std::string& from);
+  template<> std::string convert_to(const uint32_t& from);
+
+  template<> double convert_to(const std::string& from);
+  template<> std::string convert_to(const double& from);
+
   template<> Sha1 convert_to(const std::string& from);
   template<> std::string convert_to(const Sha1& from);
 
-  template<> Version convert_to(const std::string& from);
-  template<> std::string convert_to(const Version& from);
-
 }
 
 #endif
