@@ -1,5 +1,5 @@
 // Aseprite Base Library
-// Copyright (c) 2001-2013 David Capello
+// Copyright (c) 2001-2013, 2015 David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -14,8 +14,8 @@
 #include <string>
 
 namespace base {
-  
-  typedef SharedPtr<FILE> FileHandle;
+
+  typedef base::SharedPtr<FILE> FileHandle;
 
   FILE* open_file_raw(const std::string& filename, const std::string& mode);
   FileHandle open_file(const std::string& filename, const std::string& mode);
