@@ -1,5 +1,5 @@
 // Aseprite Base Library
-// Copyright (c) 2001-2013 David Capello
+// Copyright (c) 2001-2013, 2015 David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -12,7 +12,7 @@
 
 namespace base {
 
-  std::string get_pretty_memory_size(size_t memsize);
+  std::string get_pretty_memory_size(std::size_t memsize);
 
 } // namespace base
 
