@@ -1,5 +1,5 @@
 // Aseprite Base Library
-// Copyright (c) 2001-2013 David Capello
+// Copyright (c) 2001-2013, 2015 David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -8,9 +8,11 @@
 #define BASE_MEMORY_H_INCLUDED
 #pragma once
 
-void* base_malloc (size_t bytes);
-void* base_malloc0(size_t bytes);
-void* base_realloc(void* mem, size_t bytes);
+#include <cstddef>
+
+void* base_malloc (std::size_t bytes);
+void* base_malloc0(std::size_t bytes);
+void* base_realloc(void* mem, std::size_t bytes);
 void  base_free   (void* mem);
 char* base_strdup (const char* string);
 
