@@ -1,5 +1,5 @@
 // Aseprite Base Library
-// Copyright (c) 2001-2013 David Capello
+// Copyright (c) 2001-2013, 2015 David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -11,8 +11,7 @@
 #include <pthread.h>
 #include <errno.h>
 
-class base::mutex::mutex_impl
-{
+class base::mutex::mutex_impl {
 public:
 
   mutex_impl() {
