@@ -1,5 +1,5 @@
 // Aseprite Base Library
-// Copyright (c) 2001-2013 David Capello
+// Copyright (c) 2001-2013, 2015 David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -10,8 +10,7 @@
 
 #include <windows.h>
 
-class base::mutex::mutex_impl
-{
+class base::mutex::mutex_impl {
 public:
 
   mutex_impl() {
