@@ -1,5 +1,5 @@
 // Aseprite Document Library
-// Copyright (c) 2014 David Capello
+// Copyright (c) 2001-2015 David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -25,16 +25,16 @@ namespace doc {
     Context();
     virtual ~Context();
 
-    Settings* settings() const { return m_settings; }
-    void setSettings(Settings* settings);
-
     const Documents& documents() const { return m_docs; }
     Documents& documents() { return m_docs; }
 
+    Site activeSite() const;
     Document* activeDocument() const;
-    void setActiveDocument(Document* doc);
 
-    // DocumentsObserver impl
+    void notifyActiveSiteChanged();
+
+  protected:
+    virtual void onGetActiveSite(Site* site) const;
     virtual void onAddDocument(Document* doc) override;
     virtual void onRemoveDocument(Document* doc) override;
 
