@@ -1,5 +1,5 @@
 // Aseprite Document Library
-// Copyright (c) 2014 David Capello
+// Copyright (c) 2001-2015 David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -11,11 +11,12 @@
 namespace doc {
 
   class Document;
+  class Site;
 
   class ContextObserver {
   public:
     virtual ~ContextObserver() { }
-    virtual void onSetActiveDocument(Document* document) { }
+    virtual void onActiveSiteChange(const Site& site) { }
   };
 
 } // namespace doc
