@@ -1,30 +1,20 @@
-/* Aseprite
- * Copyright (C) 2001-2013  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifndef FILTERS_MEDIAN_FILTER_PROCESS_H_INCLUDED
 #define FILTERS_MEDIAN_FILTER_PROCESS_H_INCLUDED
 #pragma once
 
-#include <vector>
-
+#include "base/ints.h"
 #include "filters/filter.h"
 #include "filters/tiled_mode.h"
 
+#include <vector>
+
 namespace filters {
 
   class MedianFilter : public Filter {
