@@ -1,5 +1,5 @@
 // Aseprite Gfx Library
-// Copyright (C) 2001-2014 David Capello
+// Copyright (C) 2001-2016 David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -8,6 +8,8 @@
 #define GFX_COLOR_H_INCLUDED
 #pragma once
 
+#include "base/ints.h"
+
 namespace gfx {
 
   typedef uint32_t Color;
@@ -32,11 +34,6 @@ namespace gfx {
   inline ColorComponent getb(Color c) { return (c >> ColorBShift) & 0xff; }
   inline ColorComponent geta(Color c) { return (c >> ColorAShift) & 0xff; }
 
-  inline Color setr(Color c, ColorComponent v) { return Color((c & ~ColorRShift) | (v << ColorRShift)); }
-  inline Color setg(Color c, ColorComponent v) { return Color((c & ~ColorGShift) | (v << ColorGShift)); }
-  inline Color setb(Color c, ColorComponent v) { return Color((c & ~ColorBShift) | (v << ColorBShift)); }
-  inline Color seta(Color c, ColorComponent v) { return Color((c & ~ColorAShift) | (v << ColorAShift)); }
-
   inline bool is_transparent(Color c) { return geta(c) == 0; }
 
 } // namespace gfx
