@@ -4,6 +4,10 @@
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <gtest/gtest.h>
 
 #include "gfx/border.h"
