@@ -1,20 +1,9 @@
-/* Aseprite
- * Copyright (C) 2001-2014  David Capello
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+// Aseprite
+// Copyright (C) 2001-2016  David Capello
+//
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License version 2 as
+// published by the Free Software Foundation.
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -33,12 +22,13 @@
 #include "she/scoped_handle.h"
 #include "she/system.h"
 
+#include <clocale>
 #include <cstdlib>
 #include <ctime>
 #include <iostream>
 
-#ifdef WIN32
-#include <windows.h>
+#ifdef _WIN32
+  #include <windows.h>
 #endif
 
 namespace {
@@ -59,10 +49,16 @@ namespace {
 // Aseprite entry point. (Called from she library.)
 int app_main(int argc, char* argv[])
 {
-  // Initialize the random seed.
-  std::srand(static_cast<unsigned int>(std::time(NULL)));
+  // Initialize the locale. Aseprite isn't ready to handle numeric
+  // fields with other locales (e.g. we expect strings like "10.32" be
+  // used in std::strtod(), not something like "10,32").
+  std::setlocale(LC_ALL, "en-US");
+  ASSERT(std::strtod("10.32", nullptr) == 10.32);
 
-#ifdef WIN32
+  // Initialize the random seed.
+  std::srand(static_cast<unsigned int>(std::time(nullptr)));
+
+#ifdef _WIN32
   ::CoInitialize(NULL);
 #endif
 
