@@ -1,5 +1,5 @@
 // SHE library
-// Copyright (C) 2012-2013  David Capello
+// Copyright (C) 2012-2013, 2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -10,11 +10,11 @@
 
 namespace she {
 
-  enum Capabilities {
-    kMultipleDisplaysCapability = 1,
-    kCanResizeDisplayCapability = 2,
-    kDisplayScaleCapability = 4,
-    kMouseEventsCapability = 8,
+  enum class Capabilities {
+    MultipleDisplays = 1,
+    CanResizeDisplay = 2,
+    DisplayScale = 4,
+    GpuAccelerationSwitch = 8,
   };
 
 } // namespace she
