@@ -1,5 +1,5 @@
 // SHE library
-// Copyright (C) 2012-2014  David Capello
+// Copyright (C) 2012-2016  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -8,14 +8,12 @@
 #define SHE_H_INCLUDED
 #pragma once
 
-#include "she/clipboard.h"
 #include "she/display.h"
 #include "she/error.h"
 #include "she/event.h"
 #include "she/event_queue.h"
-#include "she/locked_surface.h"
+#include "she/font.h"
 #include "she/scoped_handle.h"
-#include "she/scoped_surface_lock.h"
 #include "she/surface.h"
 #include "she/system.h"
 
