@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2016  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -10,12 +10,14 @@
 
 #include "ui/accelerator.h"
 
-#include "base/unique_ptr.h"
+#include "base/debug.h"
+#include "base/replace_string.h"
 #include "base/split_string.h"
 #include "base/string.h"
+#include "base/unique_ptr.h"
 
-#include <allegro/keyboard.h>
 #include <cctype>
+#include <cstdlib>
 #include <string>
 #include <vector>
 
@@ -26,8 +28,25 @@
 
 namespace ui {
 
-//////////////////////////////////////////////////////////////////////
-// Accelerator
+#ifdef _WIN32
+  const char* kWinKeyName = "Win";
+#else
+  const char* kWinKeyName = "Super";
+#endif
+
+static KeyModifiers get_pressed_modifiers_from_she()
+{
+  KeyModifiers mods = kKeyNoneModifier;
+  if (she::is_key_pressed(kKeyLShift)  ) mods = KeyModifiers(int(mods) | int(kKeyShiftModifier));
+  if (she::is_key_pressed(kKeyRShift)  ) mods = KeyModifiers(int(mods) | int(kKeyShiftModifier));
+  if (she::is_key_pressed(kKeyLControl)) mods = KeyModifiers(int(mods) | int(kKeyCtrlModifier));
+  if (she::is_key_pressed(kKeyRControl)) mods = KeyModifiers(int(mods) | int(kKeyCtrlModifier));
+  if (she::is_key_pressed(kKeyAlt)     ) mods = KeyModifiers(int(mods) | int(kKeyAltModifier));
+  if (she::is_key_pressed(kKeyCommand) ) mods = KeyModifiers(int(mods) | int(kKeyCmdModifier));
+  if (she::is_key_pressed(kKeyLWin)    ) mods = KeyModifiers(int(mods) | int(kKeyWinModifier));
+  if (she::is_key_pressed(kKeyRWin)    ) mods = KeyModifiers(int(mods) | int(kKeyWinModifier));
+  return mods;
+}
 
 Accelerator::Accelerator()
   : m_modifiers(kKeyNoneModifier)
@@ -54,10 +73,12 @@ Accelerator::Accelerator(const std::string& str)
     return;
   }
 
-  std::vector<std::string> tokens;
-  base::split_string(str, tokens, "+");
-  for (std::string tok : tokens) {
-    tok = base::string_to_lower(tok);
+  std::size_t i, j;
+  for (i=0; i<str.size(); i=j+1) {
+    // i+1 because the first character can be '+' sign
+    for (j=i+1; j<str.size() && str[j] != '+'; ++j)
+      ;
+    std::string tok = base::string_to_lower(str.substr(i, j - i));
 
     if (m_scancode == kKeySpace) {
       m_modifiers = (KeyModifiers)((int)m_modifiers | (int)kKeySpaceModifier);
@@ -77,6 +98,9 @@ Accelerator::Accelerator(const std::string& str)
     else if (tok == "cmd") {
       m_modifiers = (KeyModifiers)((int)m_modifiers | (int)kKeyCmdModifier);
     }
+    else if (tok == base::string_to_lower(kWinKeyName)) {
+      m_modifiers = (KeyModifiers)((int)m_modifiers | (int)kKeyWinModifier);
+    }
 
     // Scancode
 
@@ -122,7 +146,7 @@ Accelerator::Accelerator(const std::string& str)
     else {
       // F1, F2, ..., F11, F12
       if (tok[0] == 'f' && (tok.size() <= 3)) {
-        int num = strtol(tok.c_str()+1, NULL, 10);
+        int num = std::strtol(tok.c_str()+1, NULL, 10);
         if ((num >= 1) && (num <= 12))
           m_scancode = (KeyScancode)((int)kKeyF1 + num - 1);
       }
@@ -176,15 +200,15 @@ Accelerator::Accelerator(const std::string& str)
         m_scancode = kKey8Pad;
       else if (tok == "9 pad")
         m_scancode = kKey9Pad;
-      else if (tok == "slash pad")
+      else if (tok == "/ pad" || tok == "slash pad")
         m_scancode = kKeySlashPad;
-      else if (tok == "asterisk")
+      else if (tok == "* pad" || tok == "asterisk pad" || tok == "asterisk")
         m_scancode = kKeyAsterisk;
-      else if (tok == "minus pad")
+      else if (tok == "- pad" || tok == "minus pad")
         m_scancode = kKeyMinusPad;
-      else if (tok == "plus pad")
+      else if (tok == "+ pad" || tok == "plus pad")
         m_scancode = kKeyPlusPad;
-      else if (tok == "del pad")
+      else if (tok == "del pad" || tok == "delete pad")
         m_scancode = kKeyDelPad;
       else if (tok == "enter pad")
         m_scancode = kKeyEnterPad;
@@ -219,7 +243,7 @@ bool Accelerator::isEmpty() const
 
 std::string Accelerator::toString() const
 {
-  // Same order that Allegro scancodes
+  // Same order that she::KeyScancode
   static const char* table[] = {
     NULL,
     "A",
@@ -311,7 +335,7 @@ std::string Accelerator::toString() const
     "* Pad",
     "- Pad",
     "+ Pad",
-    "Delete Pad",
+    "Del Pad",
     "Enter Pad",
     "PrtScr",
     "Pause",
@@ -325,7 +349,7 @@ std::string Accelerator::toString() const
     "KEY_COLON2",
     "Kanji",
   };
-  static size_t table_size = sizeof(table) / sizeof(table[0]);
+  static std::size_t table_size = sizeof(table) / sizeof(table[0]);
 
   std::string buf;
 
@@ -335,6 +359,10 @@ std::string Accelerator::toString() const
   if (m_modifiers & kKeyAltModifier) buf += "Alt+";
   if (m_modifiers & kKeyShiftModifier) buf += "Shift+";
   if (m_modifiers & kKeySpaceModifier) buf += "Space+";
+  if (m_modifiers & kKeyWinModifier) {
+    buf += kWinKeyName;
+    buf += "+";
+  }
 
   // Key
   if (m_unicodeChar) {
@@ -350,7 +378,7 @@ std::string Accelerator::toString() const
   return buf;
 }
 
-bool Accelerator::check(KeyModifiers modifiers, KeyScancode scancode, int unicodeChar) const
+bool Accelerator::isPressed(KeyModifiers modifiers, KeyScancode scancode, int unicodeChar) const
 {
   // Preprocess the character to be compared with the accelerator
 #ifdef PREPROCESS_KEYS
@@ -429,21 +457,22 @@ bool Accelerator::check(KeyModifiers modifiers, KeyScancode scancode, int unicod
   return false;
 }
 
-bool Accelerator::checkFromAllegroKeyArray() const
+bool Accelerator::isPressed() const
 {
-  KeyModifiers modifiers = kKeyNoneModifier;
+  KeyModifiers modifiers = get_pressed_modifiers_from_she();
 
-  if (key[KEY_LSHIFT]  ) modifiers = (KeyModifiers)((int)modifiers | (int)kKeyShiftModifier);
-  if (key[KEY_RSHIFT]  ) modifiers = (KeyModifiers)((int)modifiers | (int)kKeyShiftModifier);
-  if (key[KEY_LCONTROL]) modifiers = (KeyModifiers)((int)modifiers | (int)kKeyCtrlModifier);
-  if (key[KEY_RCONTROL]) modifiers = (KeyModifiers)((int)modifiers | (int)kKeyCtrlModifier);
-  if (key[KEY_ALT]     ) modifiers = (KeyModifiers)((int)modifiers | (int)kKeyAltModifier);
-  if (key[KEY_COMMAND ]) modifiers = (KeyModifiers)((int)modifiers | (int)kKeyCmdModifier);
-
-  return ((m_scancode == 0 || key[m_scancode]) &&
+  return ((m_scancode == 0 || she::is_key_pressed(m_scancode)) &&
           (m_modifiers == modifiers));
 }
 
+bool Accelerator::isLooselyPressed() const
+{
+  KeyModifiers modifiers = get_pressed_modifiers_from_she();
+
+  return ((m_scancode == 0 || she::is_key_pressed(m_scancode)) &&
+          (int(m_modifiers & modifiers) == m_modifiers));
+}
+
 //////////////////////////////////////////////////////////////////////
 // Accelerators
 
