@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2016  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -16,17 +16,18 @@ namespace she {
 
 namespace ui {
 
-  class ImageView : public Widget
-  {
+  class ImageView : public Widget {
   public:
-    ImageView(she::Surface* sur, int align);
+    ImageView(she::Surface* sur, int align, bool disposeSurface);
+    ~ImageView();
 
   protected:
-    void onPreferredSize(PreferredSizeEvent& ev) override;
+    void onSizeHint(SizeHintEvent& ev) override;
     void onPaint(PaintEvent& ev) override;
 
   private:
     she::Surface* m_sur;
+    bool m_disposeSurface;
   };
 
 } // namespace ui
