@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2013, 2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -9,16 +9,16 @@
 #pragma once
 
 #include "ui/entry.h"
+#include "ui/slider.h"
 
 namespace ui {
 
   class CloseEvent;
   class PopupWindow;
-  class Slider;
 
   class IntEntry : public Entry {
   public:
-    IntEntry(int min, int max);
+    IntEntry(int min, int max, SliderDelegate* sliderDelegate = nullptr);
     ~IntEntry();
 
     int getValue() const;
@@ -26,7 +26,8 @@ namespace ui {
 
   protected:
     bool onProcessMessage(Message* msg) override;
-    void onEntryChange() override;
+    void onSizeHint(SizeHintEvent& ev) override;
+    void onChange() override;
 
     // New events
     virtual void onValueChange();
@@ -36,11 +37,13 @@ namespace ui {
     void closePopup();
     void onChangeSlider();
     void onPopupClose(CloseEvent& ev);
+    void removeSlider();
 
     int m_min;
     int m_max;
+    Slider m_slider;
     PopupWindow* m_popupWindow;
-    Slider* m_slider;
+    bool m_changeFromSlider;
   };
 
 } // namespace ui
