@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -13,20 +13,13 @@
 
 namespace ui {
 
-  class Label : public Widget
-  {
+  class Label : public Widget {
   public:
     Label(const std::string& text);
 
-    gfx::Color getTextColor() const;
-    void setTextColor(gfx::Color color);
-
   protected:
-    void onPreferredSize(PreferredSizeEvent& ev) override;
+    void onSizeHint(SizeHintEvent& ev) override;
     void onPaint(PaintEvent& ev) override;
-
-  private:
-    gfx::Color m_textColor;
   };
 
 } // namespace ui
