@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -19,26 +19,27 @@ namespace ui {
   public:
     ListBox();
 
-    ListItem* getSelectedChild();
+    Widget* getSelectedChild();
     int getSelectedIndex();
 
-    void selectChild(ListItem* item);
+    void selectChild(Widget* item);
     void selectIndex(int index);
 
-    size_t getItemsCount() const;
+    std::size_t getItemsCount() const;
 
+    void makeChildVisible(Widget* item);
     void centerScroll();
     void sortItems();
 
-    Signal0<void> ChangeSelectedItem;
-    Signal0<void> DoubleClickItem;
+    base::Signal0<void> Change;
+    base::Signal0<void> DoubleClickItem;
 
   protected:
     virtual bool onProcessMessage(Message* msg) override;
     virtual void onPaint(PaintEvent& ev) override;
     virtual void onResize(ResizeEvent& ev) override;
-    virtual void onPreferredSize(PreferredSizeEvent& ev) override;
-    virtual void onChangeSelectedItem();
+    virtual void onSizeHint(SizeHintEvent& ev) override;
+    virtual void onChange();
     virtual void onDoubleClickItem();
   };
 
