@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2013, 2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -15,8 +15,7 @@ namespace ui {
 
   class Widget;
 
-  class LoadLayoutEvent : public Event
-  {
+  class LoadLayoutEvent : public Event {
   public:
     LoadLayoutEvent(Widget* source, std::istream& stream)
       : Event(source)
