@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2013, 2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -18,7 +18,7 @@ namespace ui {
   class Overlay;
 
   class OverlayManager {
-    friend class GuiSystem;     // So it can call destroyInstance() from ~GuiSystem
+    friend class UISystem;     // So it can call destroyInstance() from ~UISystem
     static OverlayManager* m_singleton;
 
     OverlayManager();
