@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2013, 2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -24,7 +24,7 @@ PaintEvent::~PaintEvent()
 {
 }
 
-Graphics* PaintEvent::getGraphics()
+Graphics* PaintEvent::graphics()
 {
   // If someone requested the graphics pointer, it means that this
   // "someone" has painted the widget.
