@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2013, 2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -15,13 +15,12 @@ namespace ui {
   class Graphics;
   class Widget;
 
-  class PaintEvent : public Event
-  {
+  class PaintEvent : public Event {
   public:
     PaintEvent(Widget* source, Graphics* graphics);
     virtual ~PaintEvent();
 
-    Graphics* getGraphics();
+    Graphics* graphics();
 
     bool isPainted() const;
 
