@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2013, 2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -28,7 +28,7 @@ namespace ui {
     DISABLE_COPYING(Property);
   };
 
-  typedef SharedPtr<Property> PropertyPtr;
+  typedef base::SharedPtr<Property> PropertyPtr;
 
 } // namespace ui
 
