@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2016  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -10,28 +10,30 @@
 
 #include "base/disable_copying.h"
 #include "base/signal.h"
+#include "base/time.h"
 
 namespace ui {
 
   class Widget;
 
-  class Timer
-  {
+  class Timer {
   public:
     Timer(int interval, Widget* owner = NULL);
     virtual ~Timer();
 
-    int getInterval() const;
+    int interval() const { return m_interval; }
     void setInterval(int interval);
 
-    bool isRunning() const;
+    bool isRunning() const {
+      return m_running;
+    }
 
     void start();
     void stop();
 
     void tick();
 
-    Signal0<void> Tick;
+    base::Signal0<void> Tick;
 
     static void pollTimers();
     static void checkNoTimers();
@@ -42,7 +44,8 @@ namespace ui {
   public:
     Widget* m_owner;
     int m_interval;
-    int m_lastTime;
+    bool m_running;
+    base::tick_t m_lastTick;
 
     DISABLE_COPYING(Timer);
   };
