@@ -1,5 +1,5 @@
 // Aseprite UI Library
-// Copyright (C) 2001-2013  David Capello
+// Copyright (C) 2001-2013, 2015  David Capello
 //
 // This file is released under the terms of the MIT license.
 // Read LICENSE.txt for more information.
@@ -11,7 +11,7 @@
 namespace ui {
 
   // Widget types.
-  enum WidgetType {
+  enum WidgetType : int {
     // Undefined (or anonymous) widget type.
     kGenericWidget,
 
